import { FormConfig } from './form-config'
export interface IColumnBaseStruct<TcontrolType extends FormConfig.TcontrolType> {
    id: string;
    desc: string;
    placeholder: string;
    controlType: TcontrolType;
}

export interface SingleTextStruct extends IColumnBaseStruct<'text'> {
    // max: number;
    // min: number;
}

export interface Textarea extends IColumnBaseStruct<'text'>  {

}

export interface SingleSelect extends IColumnBaseStruct<"select_color"> {
    options: {label: string, color: string, value: string}[];
    value: string|null;
}

export interface MultiPleSelect extends IColumnBaseStruct<"multiple_select"> {
    options: {label: string, color: string, value: string}[];
    value: string[];
}

export interface NumberValue extends IColumnBaseStruct<"number"> {
    digits: number;
    max: number;
    min: number;
    unit: string;
}

export interface SingleDate extends IColumnBaseStruct<'date'> {
    dateFormat: 'year'|'month'|'day';
}

export interface Email extends IColumnBaseStruct<'email'> {

}

export interface Telephone extends IColumnBaseStruct<'telephone'> {
    telephoneType: 1|2; // 1:手机号码 2:固定电话
}
  
type FilterStruct = any[];
export interface RelationForm extends Omit<IColumnBaseStruct<'foreign_ref'>, 'desc'> {
    comps: IColumnBaseStruct<FormConfig.TcontrolType>;
    filters: FilterStruct;
    project_id?: number; // 当是系统表时这个字段无意义 
    system_type: 1|2; // 1: 自建表 2: 系统表
    unique_no: string; // "s1017711" 表的唯一标识符
    form_id: number; // 表单Id
}

export interface FileStruct extends IColumnBaseStruct<'file'> {

}

