import { userRangeEnum, userTypeEnum, valueRangeEnum, valueTypeEnum, ruleTypeEnum, numberingEnum, cycle_typeEnum, dateFormatEnum } from './vars';

export namespace FormConfig {
    export type TcontrolType = 'text' | 'textarea' | 'number' | 'radio' | 'checkbox' | 'select' | 'select_color'
        | 'multiple_select' | 'foreign_ref' | 'child_table' | 'date' | 'date_range' | 'telephone'
        | 'email' | 'hr' | 'file' | 'owner' | 'ROLE' | 'DEPARTMENT' | 'USER' | 'task' | 'string_formula'
        | 'autocalc' | 'member' | 'autonumber' | 'defaultvalue';
    /**
     * 
     */
    export interface IPropsConfigBase<T extends TcontrolType, DefaultValue = ''> {
        controlType: T,
        label: string;
        placeholder: string;
        required: boolean;
        defaultValue: DefaultValue;
        // 预留字段
        layout: {};
    }

    /**
     * 
     */
    export interface IPropsLabelConfig extends IPropsConfigBase<'text'> {
        max: number;
        min: number;
    }

    /**
     * 
     */
    export interface IPropsPlaceholderConfig extends IPropsLabelConfig { };

    /**
     * 
     */
    export interface IPropsDescConfig extends IPropsLabelConfig { };

    interface IPropsNumberConfig extends IPropsConfigBase<'number'> {
        digits: number;
    }
    /**
     * 
     */
    export interface IPropsMaxConfig extends IPropsNumberConfig { };

    /**
     * 
     */
    export interface IPropsMinConfig extends IPropsNumberConfig { };

    /**
     * 
     */
    export interface IPropsDigits extends IPropsConfigBase<'number'> {
        digits: number;
        max: number;
        min: number;
    }
    /**
     * 
     */
    export interface IPropsOptionsConfig extends IPropsConfigBase<'checkbox'> {
        options: { label: string, value: string; color: string }[];
    }

    export interface IPropsUserConfig extends IPropsConfigBase<'member'> {
        range: userRangeEnum,       //  'company/role/project'
        choice: userTypeEnum,       //  "single/multiple"
        value_range: valueRangeEnum, // "project"
        value_key?: {id: number, name: string, role_type: number}[],
        value_type?: valueTypeEnum,   //  "current_role/current_user" 和value互斥，只能有一个
        expr: number,   
    }

    export interface IPropsAutoNumberConfig extends IPropsConfigBase<'autonumber'> {
        rules: {
            type: ruleTypeEnum,
            value: {
              numbering: numberingEnum,
              start: number, // number
              digit: number, // 1-6
              auto_increment: boolean, // true/false 是否持续递增
              increment_value: number, // 递增值，默认1 (固定)
              increment_rule: string, // 递增依据
              cycle_type: cycle_typeEnum, // "no/day/week/month/quarter/year",
            }| string | {
                expr: string, 
                format: dateFormatEnum, 
              }
        }[]
    }
    export interface IPropsAutocalcConfig extends IPropsConfigBase<'autocalc'> {

    }
}