
const { COMPONENT_TYPE } = require('../../component/component_type');
const { SPECIAL_FIELDS } = require('../component/components');
/**
 *
 * @param {*} form: object 表单配置
 * @param {*} accesses: Array 权限配置
 * @param {*} context = { roles: [ { id, type } ] }  上下文信息，如当前用户的角色列表等
 * @returns {
 *  fieldId: { readonly: boolean, visible: boolean }
 * }
 */
function parseAccess (form, accesses = [], context) {
  const accessMap = {};
  const { fields = [] } = form;
  const fieldMap = {};
  const { roles = [] } = context;

  const SPECIAL_FIELD_MAP = {};
  SPECIAL_FIELDS.forEach(item => {
    if (item.fieldGroup) SPECIAL_FIELD_MAP[item.fieldGroup] = item.id;
  });

  fields.forEach(fieldConfig => {
    let item = { readonly: false, visible: true };
    if (fieldConfig.controlType === COMPONENT_TYPE.TASK) {
      Object.keys(SPECIAL_FIELD_MAP[COMPONENT_TYPE.TASK]).forEach(f => {
        fieldMap[f] = item;
      });
    } else {
      fieldMap[fieldConfig.id] = item;
    }
  });

  accesses.forEach(access => {
    accessMap[`${access.ownerIds[0]}-${access.ownerType}`] = access.fields;
  });

  roles.forEach(roleItem => {
    const key = `${roleItem.id}-${roleItem.type}`;
    const fieldConfigs = accessMap[key] || [];

    fieldConfigs.forEach(fieldConfig => {
      const itemObj = fieldMap[fieldConfig.id];
      if (!itemObj) return;

      itemObj.visible = fieldConfig.visible || itemObj.visible ? true : false;

      if (!itemObj.visible) {
        itemObj.readonly = true;
        return;
      }

      itemObj.readonly = fieldConfig.readonly && itemObj.readonly ? true : false;
    });
  });

  return fieldMap;
}

module.exports = {
  parseAccess,
};
