
// function parse

const { COMPONENT_TYPE } = require('../component/component_type');
const { Calculator, translate, parse, CheckLex, parseFormulaStaStructure } = require('./formula2/lib/formula_main');

let SOURCE_GLOBAL_TABLE = require('./report/report').getTables();
function parseColumn (prevColumn, column, index, tableMap) {
  let { id, expr, label, controlType, source } = column;
  if (id === 'fid') return;
  if (controlType === COMPONENT_TYPE.FOREIGN_REF && !expr) {
    expr = `${source}.fid`;
  }
  const items = expr.split('.');
  const field = items.pop();
  const columnConfig = {
    setting: column,
    field,
    sourceTables: JSON.parse(JSON.stringify(items)),
    sourceTableKey: items.join('.'),
    tables: items,
    tableKey: items.join('.'),
    index,
  };

  if (!prevColumn || columnConfig.tables[0] === prevColumn.tables[0]) {

  } else if (column.controlType === COMPONENT_TYPE.MATH_FORMULA) {
    const dependColumnMap = {};
    const { s, newShortSexpr } = translate(column.expr, function (id) {
    }, function (id) {
      const ids = id.split('.');
      const names = ids.map((id, index) => {
        if (index % 2 === 0) return tableMap[id].label;
        else return tableMap[ids[index - 1]].setting.fieldMap[id].label;
      });
      const result = parseColumn(prevColumn, {
        expr: id,
      }, 0, tableMap);
      dependColumnMap[id] = result;
      return names.join('.')
    }, function (id) {
      const ids = id.split('.');
      return `${ids[ids.length - 2]}.${ids[ids.length - 1]}`;
    });
    columnConfig.dependColumnMap = dependColumnMap;
    // columnConfig.dataType = r.vtype === 'num' ? COMPONENT_TYPE.NUMBER : r.vtype === 'date' ? COMPONENT_TYPE.DATE : COMPONENT_TYPE.TEXT;
    columnConfig.tables = [];
    columnConfig.tableKey = '';
    columnConfig.sourceTables = [];
    columnConfig.sourceTableKey = '';
    columnConfig.field = dependColumnMap;
    columnConfig.expr_label = s;
    columnConfig.expr = newShortSexpr;
  } else if (prevColumn.sourceTableKey === columnConfig.sourceTableKey) {
    columnConfig.tables = prevColumn.tables;
    columnConfig.tableKey = prevColumn.tableKey;
  } else if (columnConfig.tables[0] === prevColumn.tables[prevColumn.tables.length - 1]) {
    columnConfig.tables.splice(0, 1);
    columnConfig.tables = prevColumn.tables.concat(columnConfig.tables);
    columnConfig.tableKey = columnConfig.tables.join('.');
  } else {

    const prevColumnLastTableId = prevColumn.tables[prevColumn.tables.length - 1];
    const targetTableId = columnConfig.tables[0];

    const paths = [];
    function findPath (tableId, _prev, _findLog = {}) {
      const table = tableMap[tableId];
      const findLog = JSON.parse(JSON.stringify(_findLog));
      const fieldKeys = Object.keys(table.setting.fieldMap);
      for (let i = 0; i < fieldKeys.length; i++) {
        const field = table.setting.fieldMap[fieldKeys[i]];

        if (field.source === targetTableId) {
          findLog[field.source] = true;
          const result = `${_prev ? `${_prev}.` : ''}${field.id}.${field.source}`;
          paths.push(result);
          break;
        }
        if (field.source && !findLog[field.source] && field.source !== tableId) {
          findLog[field.source] = true;
          findPath(field.source, `${_prev ? `${_prev}.` : ''}${field.id}.${field.source}`, findLog);
        }
      }
    }
    findPath(prevColumnLastTableId);
    const fieldPath = paths.sort((a, b) => a.split('.').length - b.split('.').length)[0];
    let fieldPathArr = prevColumn.tables.concat(fieldPath.split('.'));
    if (columnConfig.tables.length > 1) {
      columnConfig.tables.splice(0, 1);
      fieldPathArr = fieldPathArr.concat(columnConfig.tables);
    }
    columnConfig.tables = fieldPathArr;
    columnConfig.tableKey = columnConfig.tables.join('.');
  }
  if (columnConfig.setting.controlType !== COMPONENT_TYPE.MATH_FORMULA) {
    const expr_label = [];
    columnConfig.tables.forEach((item, index) => {
      if (item.startsWith('s')) return expr_label.push(tableMap[item].label);
      expr_label.push(tableMap[columnConfig.tables[index - 1]].setting.fieldMap[item].label);
    });

    columnConfig.expr_label = expr_label.join('.') + '.' + tableMap[columnConfig.tables[columnConfig.tables.length - 1]].setting.fieldMap[field].label;
    columnConfig.expr = columnConfig.tables.join('.') + '.' + columnConfig.field;
  }

  return columnConfig;
}

function checkIsContainHourColumn (tableName) {
  const result = Object.keys(SOURCE_GLOBAL_TABLE).find(key => {
    const table = SOURCE_GLOBAL_TABLE[key];
    if (table.table === tableName) return true;
    return false;
  });

  return result;
}

function parseConfigToSql (columns = [], conditionMap = {}, tableMap = {}, { enableLimit } = { enableLimit: false }) {
  const sqlObj = {
    columns: [
      ':current_company_id as company_id',
    ],
    from: [],
    base_where: [],
    where: [],
    order: [],
    group: [],
    limit: 'limit :page_index, :page_size',
  };
  const leftJoinTableStackMap = {};
  let hasGroupColumn = false;
  const leftJoinMap = {};

  function renderFrom (column, cindex) {
    const { setting, tables, tableKey, sourceTables, sourceTableKey, field } = column;
    const prev = columns[cindex - 1];
    if (prev && tableKey === prev.tableKey) return leftJoinMap[tableMap[prev.tables[prev.tables.length - 1]].table].name;

    const tableStackArr = [];
    let lastTableName = '';
    tables.forEach((key, index) => {
      if (key.startsWith('f')) return;
      const tableInfo = tableMap[key];

      if (cindex === 0 && index === 0) {
        sqlObj.base_where.push(`${tableInfo.table}.company_id = :current_company_id`);
      }

      tableStackArr.push(key);
      let tableName = tableInfo.table;
      lastTableName = tableName;
      const leftJoinStackKey = tableStackArr.join('.');
      if (leftJoinTableStackMap[leftJoinStackKey]) return;
      leftJoinTableStackMap[leftJoinStackKey] = true;

      const prevTableKey = tables[index - 2];
      const prevTableInfo = tableMap[prevTableKey];
      const prevTableName = prevTableInfo ? (leftJoinMap[prevTableInfo.table].name || prevTableInfo.table) : '';
      const prevField = tables[index - 1];
      const prevFieldInfo = prevTableInfo && prevTableInfo.setting.fieldMap[prevField];

      let str = '';
      if (leftJoinMap[tableInfo.table]) {
        leftJoinMap[tableInfo.table].num++;
        tableName = `${tableName}_${leftJoinMap[tableInfo.table].num}`;
        leftJoinMap[tableInfo.table].name = tableName;
      } else {
        leftJoinMap[tableInfo.table] = { num: 1, name: tableInfo.table };
      }

      if (sqlObj.from.length === 0) {
        if (tableInfo.parse_info.sql) {
          str += `(${tableInfo.parse_info.sql}) as ${tableName}`;
        } else {
          str += `${tableName}`;
        };
      } else {
        if (tableInfo.parse_info.sql) {
          str += `
            left join (${tableInfo.parse_info.sql}) as ${tableName}
              on ${tableName}.company_id = :current_company_id
          `;
        } else {
          str += `
            left join ${tableName}
              on ${tableName}.company_id = :current_company_id
          `;
        }

        if (prevFieldInfo.controlType === COMPONENT_TYPE.FOREIGN_REF) {
          const prevFieldId = prevTableInfo.isSystemTable ? prevFieldInfo.id.replace('f', '') : prevFieldInfo.id;
          str += `
            and ${tableName}.id = ${prevTableName}.${prevFieldId}
        `;
        } else if (prevFieldInfo.controlType === COMPONENT_TYPE.CHILD_TABLE) {
          const tableFieldId = tableInfo.isSystemTable ? tableInfo.setting.foreignRefMap[prevTableInfo.key].id.replace('f', '') : tableInfo.setting.foreignRefMap[prevTableInfo.key].id;
          str += `
            and ${prevTableName}.id = ${tableName}.${tableFieldId}
        `;
        }
      }

      if (checkIsContainHourColumn(tableName)) {
        sqlObj.base_where.push(`
          and ${tableName}.hour_date between :hour_begin_date and :hour_end_date
        `);
      }
      sqlObj.from.push(str);
    });
    return lastTableName;
  }

  function renderWhere (column, cindex, idKey, colKey, colKeyField, lastTableName, colId) {
    const { setting, tables, tableKey, sourceTables, sourceTableKey, field } = column;
    if (!conditionMap[setting.id] || !enableLimit) return;
    let where = [];
    conditionMap[setting.id].forEach((andArr, index) => {
      const andWhere = [];
      andArr.forEach(cond => {
        let value = typeof cond[2] === 'object' ? cond[2].value : cond[2];
        if (setting.isGroupContent) {
          andWhere.push(`${colKey} like '%${value}%'`);
        } else {
          const where = parseCalculation(colKey, cond, value)
          // andWhere.push(`${colKey} ${cond[1]} '${value}'`);
          andWhere.push(where);
        }
      });
      where.push(andWhere.join(' and '));
    });
    if (where.length > 0) {
      where = `and (${where.join(' or ')})`;
      sqlObj.where.push(where);
    }
  }

  function renderColumn (column, cindex) {
    const { setting, tables, tableKey, sourceTables, sourceTableKey, field } = column;
    const lastTableInfo = tableMap[tables[tables.length - 1]];

    if (setting.controlType === COMPONENT_TYPE.MATH_FORMULA) {
      const r = translate(column.setting.expr, function (id) {
      }, function (id) {
        const ids = id.split('.');
        const names = ids.map((id, index) => {
          if (index % 2 === 0) return tableMap[id].label;
          else return tableMap[ids[index - 1]].setting.fieldMap[id].label;
        });
        return names.join('.')
      }, function (id) {
        const ids = id.split('.');
        const tableInfo = tableMap[ids[ids.length - 2]];
        return `${tableInfo.table}.${tableInfo.isSystemTable ? ids[ids.length - 1].replace('f', '') : ids[ids.length - 1]}`;
      });
      const colKey = r.newShortSexpr;
      // sqlObj.columns.push(`null as ${id}_id`);
      sqlObj.columns.push(`${colKey} as ${setting.id}`);

      hasGroupColumn = !!r.hasGroupFn;

      if (!column.dependColumnMap || Object.keys(column.dependColumnMap).length === 0) {

      } else {
        Object.keys(column.dependColumnMap).forEach(key => {
          renderFrom(column.dependColumnMap[key], cindex);
        });
      }
      return;
    }

    // render from
    const lastTableName = renderFrom(column, cindex);
    // render column
    const idKey = lastTableInfo.isSystemTable ? `${lastTableName}.id` : `${lastTableName}.fid`;
    const colKey = lastTableInfo.isSystemTable ? `${lastTableName}.${field.replace('f', '')}` : `${lastTableName}.${field}`;
    let colKeyField = colKey;
    if (column.setting.key_field) {
      colKeyField = lastTableInfo.isSystemTable ? `${lastTableName}.${column.setting.key_field.replace('f', '')}` : `${lastTableName}.${column.setting.key_field}`;
    }

    const colId = `${setting.id}_key`;
    const colValue = setting.id;

    if (cindex === 0 && !enableLimit) {
      // sqlObj.columns.push(`row_number() over (order by ${idKey}) as fid`);
    }

    // sqlObj.columns.push(`${idKey} as ${colId}`);
    sqlObj.columns.push(`${colKey} as ${colValue}`);

    // render where
    renderWhere(column, cindex, idKey, colKey, colKeyField, lastTableName, colId);

    // sqlObj.group.push(colId);
    sqlObj.group.push(colValue);
  }

  columns.forEach((column, cindex) => {
    renderColumn(column, cindex);
  });

  const sqlArr = ['select distinct'];
  const columnCount = sqlObj.columns.length;
  sqlObj.columns.forEach((col, index) => {
    sqlArr.push(col + (columnCount - 1 === index ? '' : ','));
  });
  sqlArr.push('from');
  sqlObj.from.forEach(fromStr => {
    sqlArr.push(fromStr);
  });
  sqlArr.push('where');
  sqlObj.base_where.forEach(whereStr => {
    sqlArr.push(whereStr);
  });
  if (enableLimit) {
    sqlObj.where.forEach(whereStr => {
      sqlArr.push(whereStr);
    });
  }

  if (sqlObj.order.length > 0) {
    sqlArr.push('order by');
    const arr = [];
    sqlObj.order.forEach(orderStr => {
      arr.push(orderStr);
    });
    sqlArr.push(arr.join(', '));
  }
  if (sqlObj.group.length > 0 && hasGroupColumn) {
    sqlArr.push('group by');
    const arr = [];
    sqlObj.group.forEach(groupStr => {
      arr.push(groupStr);
    });
    sqlArr.push(arr.join(', '));
  } else {
    sqlObj.group = [];
  }

  if (enableLimit) sqlArr.push(sqlObj.limit);

  const sqlStr = sqlArr.join(' ');
  return sqlStr;
}

function parseTables (tables = []) {
  tables = JSON.parse(JSON.stringify(tables));
  const tableMap = {};
  const childMap = {};
  const tableNameMap = {};
  const GLOBAL_TABLE = JSON.parse(JSON.stringify(SOURCE_GLOBAL_TABLE));
  Object.keys(GLOBAL_TABLE).forEach(key => {
    const item = GLOBAL_TABLE[key];
    const tableKey = item.key;
    tableMap[tableKey] = item;
    tableNameMap[item.label] = item;
    item.name = item.label;
    item.parse_info = item.parse_info || { sql: '' };
    item.isSystemTable = true;
    item.setting.fieldMap = {};
    item.setting.fieldNameMap = {};
    item.setting.foreignRefMap = {};
    item.setting.fields.forEach(field => {
      item.setting.fieldMap[field.id] = field;
      item.setting.fieldNameMap[field.label] = field;
      field.vtype = field.controlType === COMPONENT_TYPE.NUMBER || field.dataType === COMPONENT_TYPE.NUMBER ? 'num' : field.controlType === COMPONENT_TYPE.DATE || field.dataType === COMPONENT_TYPE.DATE ? 'time' : '';

      if (field.controlType === COMPONENT_TYPE.FOREIGN_REF) {
        item.setting.foreignRefMap[field.source] = field;
        childMap[field.source] = childMap[field.source] || {};
        childMap[field.source][tableKey] = { label: item.label, name: item.name, source: tableKey, controlType: COMPONENT_TYPE.CHILD_TABLE };
      }
    });
  });

  tables.forEach(item => {
    const tableKey = item.key;
    tableMap[tableKey] = item;
  });

  tables.forEach(item => {
    const tableKey = item.key;
    tableMap[tableKey] = item;
    item.table = tableKey;
    item.parse_info = item.parse_info || { sql: '' };
    item.label = item.name;
    tableNameMap[item.label] = item;

    item.setting.fieldMap = {};
    item.setting.fieldNameMap = {};
    item.setting.foreignRefMap = {};
    const foreignMap = {};
    let maxNum = 0;
    item.setting.fields.forEach(field => {
      if (/^\d+$/g.test(field.id.replace('f', ''))) {
        maxNum = Math.max(maxNum, Number(field.id.replace('f', '')));
      }
    });
    for (let i = 0; i < item.setting.fields.length; i++) {
      const field = item.setting.fields[i];
      if (field.id !== 'fid' && field.controlType !== COMPONENT_TYPE.FOREIGN_REF && field.controlType !== COMPONENT_TYPE.MATH_FORMULA) {
        const expr = field.expr;

        const exprArr = expr.split('.');
        exprArr.pop();
        const newExpr = `${exprArr.join('.')}.fid`;
        if (!foreignMap[newExpr] || i - foreignMap[newExpr] > 1) {
          foreignMap[newExpr] = i;
          const source = exprArr[exprArr.length - 1];
          item.setting.fields.splice(i, 0, {
            id: `f_${exprArr.join('_')}_fid`,
            source: source,
            shown: false,
            expr: newExpr,
            controlType: COMPONENT_TYPE.FOREIGN_REF,
          });

          i--;
          continue;
        } else {
          foreignMap[newExpr] = i;
        }
      }
      item.setting.fieldMap[field.id] = field;
      item.setting.fieldNameMap[field.label] = field;
      field.vtype = field.controlType === COMPONENT_TYPE.NUMBER || field.dataType === COMPONENT_TYPE.NUMBER ? 'num' : field.controlType === COMPONENT_TYPE.DATE || field.dataType === COMPONENT_TYPE.DATE ? 'time' : 'string';

      if (field.controlType === COMPONENT_TYPE.FOREIGN_REF) {
        item.setting.foreignRefMap[field.source] = field;
        childMap[field.source] = childMap[field.source] || {};
        childMap[field.source][tableKey] = { label: item.label, name: item.name, source: tableKey, controlType: COMPONENT_TYPE.CHILD_TABLE };
      }
    }

  });

  Object.keys(childMap).forEach(tableKey => {
    Object.keys(childMap[tableKey]).forEach(childKey => {
      const item = childMap[tableKey][childKey];
      item.id = `f${Object.keys(tableMap[tableKey].setting.fieldMap).length}`;
      tableMap[tableKey].setting.fieldMap[item.id] = item;
      tableMap[tableKey].setting.fieldNameMap[item.label] = item;
      item.label = tableMap[childKey].label;
      item.name = tableMap[childKey].name;
    })
  });
  return { tableMap, tableNameMap };
}

function parseIAConfigToSql ({
  fields = [],
  conditionMap = {}
}, tables = [], { enableLimit } = { enableLimit: false }) {
  const { tableMap } = parseTables(tables);
  const parsedColumnlist = [];
  let prevColumn = null;
  fields = JSON.parse(JSON.stringify(fields));
  // 分析各列的关系链
  const foreignMap = {};
  let maxNum = 0;
  fields.forEach(field => {
    if (/^\d+$/g.test(field.id.replace('f', ''))) {
      maxNum = Math.max(maxNum, Number(field.id.replace('f', '')));
    }
  });
  for (let i = 0; i < fields.length; i++) {
    const field = fields[i];
    // if (field.id !== 'fid' && field.controlType !== COMPONENT_TYPE.FOREIGN_REF && field.controlType !== COMPONENT_TYPE.MATH_FORMULA) {
    //   const expr = field.expr;

    //   const exprArr = expr.split('.');
    //   exprArr.pop();
    //   // const newExpr = `${exprArr.join('.')}.fid`;
    //   if (!foreignMap[newExpr] || i - foreignMap[newExpr] > 1) {
    //     foreignMap[newExpr] = i;

    //     fields.splice(i, 0, {
    //       id: `f_${exprArr.join('_')}_fid`,
    //       source: exprArr[exprArr.length - 1],
    //       shown: false,
    //       expr: newExpr,
    //       controlType: COMPONENT_TYPE.FOREIGN_REF,
    //     });

    //     i--;
    //     continue;
    //   } else {
    //     foreignMap[newExpr] = i;
    //   }
    // }
    const result = parseColumn(prevColumn, field, i, tableMap);
    if (!result) continue;
    if (result.setting.controlType !== COMPONENT_TYPE.MATH_FORMULA) {
      prevColumn = result;
    }
    parsedColumnlist.push(result);
  }

  const sql = parseConfigToSql(parsedColumnlist, conditionMap, tableMap, { enableLimit });
  return sql;
}

function checkHasHourTable ({ fields }, tables = []) {
  // const { tableMap } = parseTables(tables);
  // let prevColumn = null;
  // // 分析各列的关系链
  // for (let i = 0; i < fields.length; i++) {
  //   const result = parseColumn(prevColumn, fields[i], i, tableMap);
  //   if (!result) continue;
  //   if (!result || result.setting.controlType === COMPONENT_TYPE.MATH_FORMULA) {

  //   } else {
  //     prevColumn = result;
  //   }
  //   if (result.tables.filter(t => t === SOURCE_GLOBAL_TABLE.HOUR.key)) {
  //     return true;
  //   }
  // }
  // return false;
  return true;
}

/**
 *
 * @param {*} expr string
 * @param {*} tables 自定义报表列表
 * @returns
 */
function parseExpr (expr, tables = []) {
  expr += '';
  const { tableMap, tableNameMap } = parseTables(tables);
  const r = parse(expr, function (id) {
    const ids = id.split('.');
    const field = tableMap[ids[ids.length - 2]].setting.fieldMap[ids[ids.length - 1]];
    return {
      vtype: field.controlType === COMPONENT_TYPE.NUMBER || field.dataType === COMPONENT_TYPE.NUMBER ? 'num' : field.controlType === COMPONENT_TYPE.DATE || field.dataType === COMPONENT_TYPE.DATE ? 'date' : 'string',
      table: tableMap[ids[ids.length - 2]].key,
    };
  }, function (name) {
    const names = name.split('.');
    const ids = names.map((name, index) => {
      if (index % 2 === 0) return tableNameMap[name].key;
      else return tableNameMap[names[index - 1]].setting.fieldNameMap[name].id;
    });
    return ids.join('.')
  }, function (id) {
    const ids = id.split('.');
    return `${ids[ids.length - 2]}.${ids[ids.length - 1]}`;
  });
  return {
    expr: r.sexpr,
    dataType: r.vtype === 'num' ? COMPONENT_TYPE.NUMBER : r.vtype === 'date' ? COMPONENT_TYPE.DATE : COMPONENT_TYPE.TEXT,
    err: r.err,
  };
}

/**
 * 计算符转换sql where语句
 */
function parseCalculation (colKey, cond, value) {
  let result = `${colKey} = '${value}'`;
  switch (cond[1]) {
    case '=':
        result = `${colKey} = '${value}'`;
      break;
    case '!=':
      result = `${colKey} <> '${value}'`;
      break;
    case 'be_null':
      result = `${colKey} is null`;
      break;
    case 'not_be_null':
      result = `${colKey} is not null`;
      break;
    case '<':
      result = `${colKey} < '${value}'`;
      break;
    case '>':
      result = `${colKey} > '${value}'`;
      break;
    case '<=':
      result = `${colKey} <= '${value}'`;
      break;
    case '>=':
      result = `${colKey} >= '${value}'`;
      break;
    case 'before':
      result = `${colKey} < '${value}'`;
      break;
    case 'after':
      result = `${colKey} > '${value}'`;
      break;
    case 'between':
      result = `${colKey} between '${value[0]}' and '${value[1]}'`;
      break;
    case 'not_between':
      result = `${colKey} not between '${value[0]}' and '${value[1]}'`;
      break;
    default: result = `${colKey} = '${value}'`;
      break;
  }

  return result;
}

module.exports = { parseIAConfigToSql, parseExpr, checkHasHourTable };

