const { COMPONENT_TYPE, DATASOURCE } = require("../../../component/components")

const {
  CHILD_TABLE,
  FOREIGN_REF,
  NUMBER,
  TEXT,
  MATH_FORMULA,
  DATE,
  SELECT,
  DEPARTMENT,
  USER,
} = COMPONENT_TYPE

const WIDE_DEPARTMENT = {
  key: "swide_department", // 表名
  table: "analysis_wide_hour_by_department_records", // 实际表
  label: "部门", // 表名
  level: 1, // 层级，是否外层显示
  setting: {
    fields: [
      // 基础信息
      {
        id: "fid", // 别名
        label: "id", // 名称
        shown: false, // 是否展示
        controlType: NUMBER,
      },
      {
        id: "fdepartment_name_key",
        label: "部门id",
        shown: false,
        controlType: NUMBER,
      },
      {
        id: "fdepartment_name",
        label: "部门名称",
        key_field: "fdepartment_name_key",
        shown: true,
        controlType: DEPARTMENT,
      },
      {
        id: "fuser_name_key",
        label: "人员id",
        shown: false,
        controlType: NUMBER,
      },
      {
        id: "fuser_name",
        label: "人员姓名",
        key_field: "fuser_name_key",
        shown: true,
        controlType: USER,
      },
      {
        id: "fproject_count",
        label: "项目数",
        shown: true,
        controlType: NUMBER,
      },
      {
        id: "fchild_project_count",
        label: "中心数",
        shown: true,
        controlType: NUMBER,
      },
      {
        id: "fproject_seq_no_key",
        label: "项目id",
        shown: false,
        controlType: NUMBER,
        group: "project",
      },
      {
        id: "fproject_seq_no",
        label: "项目编号",
        key_field: "fproject_seq_no_key",
        shown: true,
        dataSource: {
          type: DATASOURCE.API,
          url: "",
          method: "getProjectSelect",
          params: {},
          result: {
            id_key: "id",
            value_key: "seq_no",
          },
        },
        controlType: SELECT,
        group: "project",
      },
      {
        id: "fproject_current_milestone",
        label: "项目阶段",
        shown: true,
        controlType: TEXT,
      },
      {
        id: "fsponsor",
        label: "申办方",
        shown: true,
        controlType: TEXT,
      },
      {
        id: "fproject_role_name",
        label: "项目角色",
        isGroupContent: true,
        shown: true,
        dataSource: {
          type: DATASOURCE.API_METHOD,
          method: "getSystemRoles",
          result: {
            id_key: "id",
            value_key: "name",
          },
        },
        controlType: SELECT,
      },
      {
        id: "fproduct_name",
        label: "产品名称",
        shown: true,
        controlType: TEXT,
      },
      {
        id: "findication_comment",
        label: "适应症",
        shown: true,
        controlType: TEXT,
      },
      {
        id: "fchild_project_name_key",
        label: "中心id",
        shown: false,
        source: "main_projects",
        controlType: SELECT,
      },
      {
        id: "fchild_project_name",
        label: "中心名称",
        shown: true,
        controlType: TEXT,
      },
      {
        id: "fchild_project_number",
        label: "中心编号",
        shown: true,
        controlType: TEXT,
      },
      {
        id: "fchild_project_department_name",
        label: "中心所处办事处",
        shown: true,
        controlType: TEXT,
      },
      {
        id: "fchild_project_province",
        label: "中心所在省",
        shown: true,
        controlType: TEXT,
      },
      {
        id: "fchild_project_city",
        label: "中心所在市",
        shown: true,
        controlType: TEXT,
      },
      {
        id: "fchild_project_status",
        label: "中心状态",
        shown: true,
        controlType: TEXT,
      },
      {
        id: "fchild_project_current_milestone",
        label: "中心阶段",
        shown: true,
        controlType: TEXT,
      },
      // {
      //   id: "fchild_project_role_name",
      //   label: "中心角色",
      //   isGroupContent: true,
      //   shown: true,
      //   dataSource: {
      //     type: DATASOURCE.API_METHOD,
      //     method: "getSystemRoles",
      //     result: {
      //       id_key: "id",
      //       value_key: "name",
      //     },
      //   },
      //   controlType: SELECT,
      // },
      {
        id: "fchild_project_standard_time",
        label: "中心标准工时",
        shown: true,
        controlType: NUMBER,
      },
      {
        id: "fchild_project_plan_time",
        label: "中心计划工时",
        shown: true,
        controlType: NUMBER,
      },
      {
        id: "fchild_project_actual_time",
        label: "中心实际工时",
        shown: true,
        controlType: NUMBER,
      },
      {
        id: "fhour_date",
        label: "时间",
        shown: false,
        controlType: DATE,
      },
      {
        id: "fstandard_time",
        label: "标准工时",
        shown: true,
        controlType: NUMBER,
      },
      {
        id: "fplan_time",
        label: "计划工时",
        shown: true,
        controlType: NUMBER,
      },
      {
        id: "factual_time",
        label: "实际工时",
        shown: true,
        controlType: NUMBER,
      },
    ],
  },
  parse_info: {
    sql: `
      select
        id,
        company_id,
        department_name_key,
        department_name,
        analysis_wide_hour_by_department_records.user_name_key,
        user_name,
        project_count,
        child_project_count,
        project_seq_no_key,
        project_seq_no,
        project_current_milestone,
        sponsor,
        project_role_name,
        product_name,
        indication_comment,
        child_project_name_key,
        child_project_name,
        child_project_number,
        child_project_department_name,
        child_project_province,
        child_project_city,
        child_project_status,
        child_project_current_milestone,
        child_project_standard_time,
        child_project_plan_time,
        child_project_actual_time,
        hour_date,
        s.standard_time,
        s.plan_time,
        s.actual_time,
        0 as is_deleted
      from analysis_wide_hour_by_department_records
        left join
          ( select user_id as user_name_key,
              sum(standard_time) as standard_time,
              sum(project_plan_time) as plan_time,
              sum(actual_time) as actual_time
            from project_working_times
            where begin_date between :hour_begin_date and :hour_end_date
              and is_deleted = 0 and is_actived = 1
            group by user_id
          ) as s
          on s.user_name_key = analysis_wide_hour_by_department_records.user_name_key
    `,
  },
}

module.exports = WIDE_DEPARTMENT
