const { COMPONENT_TYPE } = require("./component_type")
const { list: formulaList } = require("../formula/lib/formula_funcs");

const PROPS = {
  label: {
    label: "标题",
    controlType: COMPONENT_TYPE.TEXT,
    placeholder: "标题",
    max: 50,
    min: 1,
    required: true,
    value: "",
    layout: {},
    access: {
      visible: true,
      readonly: false,
    },
  },
  placeholder: {
    controlType: COMPONENT_TYPE.TEXT,
    canModify: true,
    label: "框内字段说明",
    placeholder: "框内字段说明",
    defaultValue: "",
    max: 20,
    min: 0,
  },
  desc: {
    controlType: COMPONENT_TYPE.TEXTAREA,
    canModify: true,
    label: "框外字段说明",
    placeholder: "框外字段说明",
    defaultValue: "",
    max: 100,
    min: 0,
  },
  max: {
    controlType: COMPONENT_TYPE.NUMBER,
    digits: 0,
    label: "最大值",
    canModify: false,
    placeholder: "最大值",
    defaultValue: 0,
    required: false,
    layout: {},
  },
  min: {
    controlType: COMPONENT_TYPE.NUMBER,
    digits: 0,
    label: "最小值",
    canModify: false,
    placeholder: "最小值",
    defaultValue: 0,
    required: false,
    layout: {},
  },
  digits: {
    controlType: COMPONENT_TYPE.NUMBER,
    digits: 0,
    label: "小数点位数",
    canModify: true,
    placeholder: "小数位数",
    defaultValue: 0,
    max: 4,
    min: 0,
    required: false,
    layout: {},
  },
  required: {
    controlType: COMPONENT_TYPE.CHECKBOX,
    label: "验证",
    placeholder: "",
    canModify: true,
    value: [false],
    options: [{ label: "必填", value: false }],
    required: false,
  },
  unit: {
    controlType: COMPONENT_TYPE.TEXT,
    label: "单位",
    canModify: true,
    placeholder: "单位",
    defaultValue: "个",
    max: 10,
    min: 0,
    required: false,
  },
  dateFormat: {
    controlType: COMPONENT_TYPE.SELECT,
    label: "格式",
    canModify: true,
    placeholder: "格式",
    defaultValue: "",
    required: false,
    options: [
      { label: "年", value: "YYYY" },
      { label: "年-月", value: "YYYY-MM" },
      { label: "年-月-日", value: "YYYY-MM-DD" },
      { label: "年-月-日 时:分:秒", value: "YYYY-MM-DD HH:mm:ss" },
    ],
  },
  options: {
    controlType: COMPONENT_TYPE.CHECKBOX,
    multiple: true,
    label: "选项",
    canModify: true,
    placeholder: "",
    options: [
      { label: "选择项", value: "1", color: "" },
      { label: "选择项", value: "2", color: "" },
      { label: "选择项", value: "3", color: "" },
    ],
    required: false,
    access: {
      visible: true,
      readonly: false,
    },
  },
  // TODO 2.0.2
  // rules: {
  //   controlType: COMPONENT_TYPE.AUTO_NUMBER,
  //   canModify: true
  // },
  // expr: {
  //   controlType: COMPONENT_TYPE.MATH_FORMULA,
  //   canModify: true,
  //   formulaList,
  //   required: false,
  //   defaultValue: '',
  //   layout: {},
  // },
  // expr: {
  //   controlType: COMPONENT_TYPE.TEXT,
  //   label: "公式",
  //   canModify: false, // 公式暂时不做，先隐藏
  //   placeholder: "请输入公式",
  //   defaultValue: "",
  //   required: false,
  //   layout: {},
  // },
}
module.exports = PROPS
