const { COMPONENT_TYPE } = require("./component_type")
const formulaList = require("../formula/lib/formula_funcs");


//  基本类型
const BASE_PROPS = {
  label: {
    controlType: COMPONENT_TYPE.TEXT,
    label: "标题",
    placeholder: "标题",
    defaultValue: "",
    rules: [
      { required: true, message: '请输入标题' },
      {type: 'string', max: 50, min: 1}
    ]
  },
  placeholder: {
    controlType: COMPONENT_TYPE.TEXT,
    label: "框内字段说明",
    placeholder: "框内字段说明",
    defaultValue: "",
    rules: [
      { type: 'string', max: 20, min: 0 }
    ]
  },
  desc: {
    controlType: COMPONENT_TYPE.TEXTAREA,
    label: "框外字段说明",
    placeholder: "框外字段说明",
    defaultValue: "",
    rules: [
      { type: 'string', max: 100, min: 0}
    ]
  },
  max: {
    controlType: COMPONENT_TYPE.NUMBER,
    digits: 0,
    label: "最大值",
    placeholder: "最大值",
    defaultValue: 0,
    rules: [{
      required: false
    }]
  },
  min: {
    controlType: COMPONENT_TYPE.NUMBER,
    digits: 0,
    label: "最小值",
    placeholder: "最小值",
    defaultValue: 0,
    rules:[{
      required: false
    }]
  },
  digits: {
    controlType: COMPONENT_TYPE.NUMBER,
    digits: 0,
    label: "小数点位数",
    placeholder: "小数位数",
    defaultValue: 0,
    rules: [
      { type: 'number', max: 4, min: 0 }]
  },
  options: {
    controlType: COMPONENT_TYPE.CHECKBOX,
    multiple: true,
    label: "选项",
    placeholder: "",
    options: [
      { label: "选择项", value: "1", color: "" },
      { label: "选择项", value: "2", color: "" },
      { label: "选择项", value: "3", color: "" },
    ],
  },
  required: {
    controlType: COMPONENT_TYPE.CHECKBOX,
    label: "验证",
    placeholder: "",
    defaultValue: [],
    options: [{ label: "必填", value: false }],
    rules: [
      { required: false }
    ]
  },
  unit: {
    controlType: COMPONENT_TYPE.TEXT,
    label: "单位",
    placeholder: "单位",
    defaultValue: "个",
    rules:[
      { type: 'string', max: 10, min: 0 }
    ]
  },
  dateFormat: {
    controlType: COMPONENT_TYPE.SELECT,
    label: "格式",
    placeholder: "格式",
    defaultValue: "",
    options: [
      { label: "年", value: "YYYY" },
      { label: "月", value: "MM" },
      { label: "日", value: "DD" },
      { label: "年-月", value: "YYYY-MM" },
      { label: "年-月-日", value: "YYYY-MM-DD" },
      // { label: "年-月-日 时:分:秒", value: "YYYY-MM-DD HH:mm:ss" },
    ],
    rules: [{
      required: false,
    }]
  },
  dateDefaultValue: {
    controlType: COMPONENT_TYPE.SELECT,
    label: "默认值",
    placeholder: "请选择",
    defaultValue: "no",
    options: [
      { label: "不设置", value: "no" },
      { label: "行的创建日期", value: "create" },
    ],
    rules: [{
      required: false
    }]
  },
  telephoneType: {
    controlType: COMPONENT_TYPE.RADIO,
    label: "类型",
    placeholder: "类型",
    defaultValue: 1,
    options: [
      { label: "手机号码", value: 1 },
      { label: "座机号码", value: 2 },
    ],
    rules: [{
      required: true
    }]
  },
};

//  复杂类型
const CUSTOM_PROPS = {
  rules: {
    controlType: COMPONENT_TYPE.AUTO_NUMBER,
  },
  formula: {
    controlType: COMPONENT_TYPE.MATH_FORMULA,
    formulaList,
    defaultValue: '',
    rules:[{
      required: false
    }]
  },
  memberRange: {
    controlType: COMPONENT_TYPE.MEMBER
  },
  selectProject: {
    controlType: COMPONENT_TYPE.SELECT,
    label: "选择项目",
    placeholder: "请选择",
    dataSource: {},
    options: [],
    rules:[{
      required: false
    }]
  },

  selectUsers: {
    controlType: COMPONENT_TYPE.SELECT,
    label: "选择用户返回",
    placeholder: "请选择",
    required: false,
    dataSource: {},
    options: [],
  },
  ownerSelectType: {
    controlType: COMPONENT_TYPE.RADIO,
    label: "类型",
    placeholder: "类型",
    defaultValue: 1,
    options: [
      { label: "单个成员", value: 1 },
      { label: "多个成员", value: 2 },
    ],
    rules:[{
      required: true
    }]
  },
  foreignRefTables: {
    controlType: COMPONENT_TYPE.SELECT,
    label: "选择表单",
    placeholder: "请选择",
    dataSource: {},
    options: [],
    rules:[{
      required: true
    }]
  },
  formType: {
    controlType: COMPONENT_TYPE.SELECT,
    label: "表单类型",
    placeholder: "请选择",
    defaultValue: "bookProject",
    options: [
      { label: "本项目", value: "bookProject" },
      { label: "其他项目", value: "otherProject" },
      { label: "系统表", value: "system" },
    ],
    rules:[{
      required: false
    }]
  },
  associatedField: {
    controlType: COMPONENT_TYPE.SELECT,
    label: "选择字段关联",
    placeholder: "请选择",
    dataSource: {},
    options: [],
    rules:[{
      required: false
    }]
  },
  contentAassociation: {
    controlType: COMPONENT_TYPE.RADIO,
    label: "内容关联",
    placeholder: "内容关联",
    defaultValue: 1,
    required: false,
    options: [
      { label: "双向关联", value: 1 },
      { label: "AB关联", value: 2 },
      { label: "BA关联", value: 3 },
      { label: "无关联", value: 4 },
    ],
    rules:[{
      required: false
    }]
  },
  idTYPE: {
    controlType: COMPONENT_TYPE.NUMBER,
    label: "编号",
    placeholder: "编号",
    defaultValue: "",
      min: 1,
      required: true,
  },
  auxiliarylogo: {
    controlType: COMPONENT_TYPE.CHECKBOX,
    label: "辅助标识",
    placeholder: "辅助标识",
    options: [{ label: "辅助标识", value: 1, color: "" }],
    defaultValue: "",
  },
}

const PROPS = {
  ...BASE_PROPS,
  ...CUSTOM_PROPS
}
module.exports = PROPS
