const {
  COMPONENT_TYPE,
  COMPONENT_RENDER_TYPE,
  DATASOURCE,
} = require("./component_type")
const PROPS = require("./component_prop")

const cloneProp = function (obj, extend) {
  return Object.assign(JSON.parse(JSON.stringify(obj)), extend)
}

const COMPONENT_GROUP = {
  BASE: "基础组件",
  HIGH_LEVEL: "高级组件",
  TASK: "任务套件",
}

/**
 * label：组件名称
 * controlType：控件类型
 * group: 所在组
 * props: 编辑配置中需要配置的form表单数据
 * rules: 限制规则
 */
const COMPONENTS = [
  {
    label: "单行文本",
    controlType: COMPONENT_TYPE.TEXT,
    group: COMPONENT_GROUP.BASE,
    icon: "iconwenbendanhangwenben",
    compDesc: "可以输入一行文字，最多100汉字",
    props: {
      label: { ...PROPS.label },
      // required: cloneProp(PROPS.required),
      placeholder: cloneProp(PROPS.placeholder),
      desc: cloneProp(PROPS.desc),
    },
    rules: [{ type: "string", max: 100, min: 0 }],
  },
  {
    label: "多行文本",
    controlType: COMPONENT_TYPE.TEXTAREA,
    group: COMPONENT_GROUP.BASE,
    icon: "iconwenbenduohangwenben",
    compDesc: "可以输入多行文字，最多300汉字",
    props: {
      label: { ...PROPS.label },
      // required: cloneProp(PROPS.required),
      placeholder: cloneProp(PROPS.placeholder),
      desc: cloneProp(PROPS.desc),
    },
    rules: [{ type: "string", max: 300, min: 0 }],
  },
  {
    label: "下拉单选",
    controlType: COMPONENT_TYPE.SELECTCOLOR,
    group: COMPONENT_GROUP.BASE,
    icon: "iconxialadanxuankuang",
    compDesc: "可以选择一个下拉值",
    props: {
      label: { ...PROPS.label },
      options: cloneProp(PROPS.options, { multiple: false }),
      desc: cloneProp(PROPS.desc),
      placeholder: cloneProp(PROPS.placeholder),
    },
  },
  {
    label: "下拉复选",
    controlType: COMPONENT_TYPE.MULTIPLE_SELECT,
    group: COMPONENT_GROUP.BASE,
    icon: "iconxialaduoxuan",
    compDesc: "可以选择多个下拉值",
    props: {
      label: { ...PROPS.label },
      options: cloneProp(PROPS.options, { multiple: true }),
      desc: cloneProp(PROPS.desc),
      placeholder: cloneProp(PROPS.placeholder),
    },
  },
  {
    label: "数字",
    controlType: COMPONENT_TYPE.NUMBER,
    group: COMPONENT_GROUP.BASE,
    icon: "iconshuzi",
    compDesc: "只可输入数字",
    props: {
      label: { ...PROPS.label },
      digits: { ...PROPS.digits }, //cloneProp(PROPS.digits, { label: '小数位数', rules: [{ max: 9999999999, min: 0 }]}),
      unit: cloneProp(PROPS.unit),
      placeholder: cloneProp(PROPS.placeholder),
      desc: cloneProp(PROPS.desc),
    },
  },
  {
    label: "单日期",
    controlType: COMPONENT_TYPE.DATE,
    group: COMPONENT_GROUP.BASE,
    icon: "iconbiaodan_riqi",
    compDesc: "可以设置单个日期",
    props: {
      label: { ...PROPS.label },
      dateFormat: cloneProp(PROPS.dateFormat),
      dateDefaultValue: cloneProp(PROPS.dateDefaultValue),
      desc: cloneProp(PROPS.desc),
    },
  },
  // {
  //   label: "范围日期",
  //   controlType: COMPONENT_TYPE.DATE_RANGE,
  //   group: COMPONENT_GROUP.BASE,
  //   icon: "iconrili",
  //   compDesc: "可以设置范围日期",
  //   props: {
  //     label: { ...PROPS.label },
  //     // required: cloneProp(PROPS.required),
  //     dateFormat: cloneProp(PROPS.dateFormat),
  //     desc: cloneProp(PROPS.desc),
  //   },
  //   fieldValidateFn: function (value, config) {
  //     // return config.require || value !== null ? /^\d+$/g.test(value + '') : true;
  //   },
  // },
  {
    label: "电话",
    controlType: COMPONENT_TYPE.TELEPHONE,
    group: COMPONENT_GROUP.BASE,
    icon: "icondianhua",
    compDesc: "可以输入手机号或座机，并自动验证格式",
    props: {
      label: { ...PROPS.label },
      telephoneType: cloneProp(PROPS.telephoneType),
      placeholder: cloneProp(PROPS.placeholder),
      desc: cloneProp(PROPS.desc),
    },
    fieldValidateFn: function (value, config) {
      let reg = "^1[345678]\\d{9}$" // 手机号
      // let reg1 = "\\d{3}-\\d{8}|\\d{4}-\\{7,8}"
      let obj = {
        pattern: config.telephoneType === 1 ? reg : null,
        message: "输入格式不正确",
      }
      return obj
    },
  },
  {
    label: "邮箱",
    controlType: COMPONENT_TYPE.EMAIL,
    group: COMPONENT_GROUP.BASE,
    icon: "iconyouxiang",
    compDesc: "可以输入邮箱，并自动验证",
    props: {
      label: { ...PROPS.label },
      placeholder: cloneProp(PROPS.placeholder),
      desc: cloneProp(PROPS.desc),
    },
    fieldValidateFn: function (value, config) {
      let reg = "^[a-zA-Z0-9_-]+@[a-zA-Z0-9_-]+(\\.[a-zA-Z0-9_-]+)+$"
      let obj = { pattern: reg, message: "输入格式不正确" }
      return obj
    },
  },
  // {
  //   label: "分割线",
  //   controlType: COMPONENT_TYPE.HR,
  //   group: COMPONENT_GROUP.BASE,
  //   icon: "iconfengexian",
  //   compDesc: "可以作为段落信息标题",
  //   props: {
  //     label: { ...PROPS.label },
  //   },
  // },
  {
    label: "文件",
    controlType: COMPONENT_TYPE.FILE,
    group: COMPONENT_GROUP.HIGH_LEVEL,
    icon: "iconattachment",
    compDesc: "可以上传图片，文档，表单等文件",
    props: {
      label: { ...PROPS.label },
      desc: cloneProp(PROPS.desc),
    },
    // fieldValidateFn: function (value, config) {
    // return config.require || value !== null ? /^\d+$/g.test(value + '') : true;
    // },
  },
  // {
  //   label: "成员",
  //   controlType: COMPONENT_TYPE.OWNER,
  //   group: COMPONENT_GROUP.HIGH_LEVEL,
  //   icon: "iconchengyuan",
  //   compDesc: "可以选择项目中的成员",
  //   props: {
  //     label: { ...PROPS.label },
  //     // required: cloneProp(PROPS.required),
  //     ownerSelectType: cloneProp(PROPS.ownerSelectType),
  //     placeholder: cloneProp(PROPS.placeholder),
  //     desc: cloneProp(PROPS.desc),
  //   },
  // },
  {
    label: "关联他表",
    controlType: COMPONENT_TYPE.FOREIGN_REF,
    group: COMPONENT_GROUP.HIGH_LEVEL,
    compDesc: "选择关联一个多行表，可以从中引用或创建记录",
    props: {
      label: { ...PROPS.label },
    },
  },
  {
    label: "自动编号",
    controlType: COMPONENT_TYPE.AUTO_NUMBER,
    group: COMPONENT_GROUP.HIGH_LEVEL,
    compDesc: "自动编号描述",
    ruleDesc: "编号规则",
    props: {
      label: { ...PROPS.label },
      rules: cloneProp(PROPS.rules),
      desc: cloneProp(PROPS.desc),
    },
  },
  {
    label: "用户",
    controlType: COMPONENT_TYPE.MEMBER,
    group: COMPONENT_GROUP.HIGH_LEVEL,
    props: {
      label: { ...PROPS.label },
      userRange: cloneProp(PROPS.memberRange, { label: "用户范围" }),
      desc: cloneProp(PROPS.desc),
    },
  },
  {
    label: "公式",
    controlType: COMPONENT_TYPE.MATH_FORMULA,
    group: COMPONENT_GROUP.HIGH_LEVEL,
    props: {
      label: { ...PROPS.label },
      formula: { ...PROPS.formula },
      desc: cloneProp(PROPS.desc),
    },
  },
]

const SPECIAL_FIELDS = [
  {
    id: "fid",
    editInConsole: false,
  },
  {
    id: "fcreatedat",
    editInConsole: false,
  },
  {
    id: "fupdatedat",
    editInConsole: false,
  },
  {
    id: "fdeletedat",
    editInConsole: false,
  },
  {
    id: "fisdeleted",
    editInConsole: false,
  },
  {
    id: "fisactived",
    editInConsole: false,
  },
  {
    id: "fownerid",
    editInConsole: false,
  },
  {
    id: "ftask",
    editInConsole: true,
  },
  {
    id: "ftaskowner",
    fieldGroup: "task",
    editInConsole: false,
  },
  {
    id: "ftaskparticipant",
    fieldGroup: "task",
    editInConsole: false,
  },
  {
    id: "ftaskworkers",
    editInConsole: false,
  },
  {
    id: "ftaskstartedat",
    fieldGroup: "task",
    editInConsole: false,
  },
  {
    id: "ftaskendedat",
    fieldGroup: "task",
    editInConsole: false,
  },
  {
    id: "ftaskstatus",
    fieldGroup: "task",
    editInConsole: false,
  },
  {
    id: "ftaskstandardtime",
    fieldGroup: "task",
    editInConsole: false,
  },
  {
    id: "ftaskactualtime",
    fieldGroup: "task",
    editInConsole: false,
  },
  {
    id: "ftaskplantime",
    fieldGroup: "task",
    editInConsole: false,
  },
]

const FIELD_PROPS = {
  id: "f1",
  editInConsole: true,
  label: "",
  expr: "",
}
module.exports = {
  COMPONENT_TYPE,
  COMPONENT_RENDER_TYPE,
  COMPONENTS,
  SPECIAL_FIELDS,
  DATASOURCE,
}
