
const { calcUserOwnerTypes } = require('./form_access_parser');
const { COMPONENT_TYPE } = require('../component/component_type');
/**
{
  [UUID.SingleLineText]: [
    { label: '为空', value: 'be_null' },
    { label: '不为空', value: 'not_be_null' },
  ],
  [UUID.MultilineText]: [
    { label: '为空', value: 'be_null' },
    { label: '不为空', value: 'not_be_null' },
  ],
  [UUID.Mailbox]: [
    { label: '为空', value: 'be_null' },
    { label: '不为空', value: 'not_be_null' },
  ],
  [UUID.Phone]: [
    { label: '为空', value: 'be_null' },
    { label: '不为空', value: 'not_be_null' },
  ],
  [UUID.DropDownRadio]: [
    { label: '等于', value: '=' },
    { label: '不等于', value: '!=' },
    { label: '为空', value: 'be_null' },
    { label: '不为空', value: 'not_be_null' },
  ],
  [UUID.DropDownCheckBox]: [
    { label: '等于', value: '=' },
    { label: '不等于', value: '!=' },
    { label: '为空', value: 'be_null' },
    { label: '不为空', value: 'not_be_null' },
  ],
  [UUID.RadioButton]: [
    { label: '等于', value: '=' },
    { label: '不等于', value: '!=' },
  ],
  [UUID.CheckBox]: [
    { label: '等于', value: '=' },
    { label: '不等于', value: '!=' },
  ],
  [UUID.Number]: [
    { label: '等于', value: '=' },
    { label: '不等于', value: '!=' },
    { label: '小于', value: '<' },
    { label: '大于', value: '>' },
    { label: '小于等于', value: '<=' },
    { label: '大于等于', value: '>=' },
    { label: '在范围内', value: 'between' },
    { label: '不在范围内', value: 'not_between' },
    { label: '为空', value: 'be_null' },
    { label: '不为空', value: 'not_be_null' },
  ],
  [UUID.SingleDate]: [
    { label: '等于', value: '=' },
    { label: '不等于', value: '!=' },
    { label: '早于', value: 'before' },
    { label: '晚于', value: 'after' },
    { label: '为空', value: 'be_null' },
    { label: '不为空', value: 'not_be_null' },
  ],
  [UUID.RangeDate]: [
    { label: '等于', value: '=' },
    { label: '不等于', value: '!=' },
    { label: '早于', value: 'before' },
    { label: '晚于', value: 'after' },
    { label: '为空', value: 'be_null' },
    { label: '不为空', value: 'not_be_null' },
  ],
  [UUID.Member]: [
    { label: '等于', value: '=' },
    { label: '不等于', value: '!=' },
    { label: '为空', value: 'be_null' },
    { label: '不为空', value: 'not_be_null' },
  ],
  [UUID.DividingLine]: [],
  [UUID.File]: [
    { label: '为空', value: 'be_null' },
    { label: '不为空', value: 'not_be_null' },
  ],
}
 */
/**
 * [
 *  [['f1', '=', 1], ['f2', '=', '3']],
 *  [['f1', '=', 1], ['f2', '=', '3']]
 * ]
 *
 * function () {
 *  return (f1 === 1 && f2 === 3) || (f1 === 1 && f2 === 3)
 * }
 * @param {*} item
 * @returns
 */

function isCondition (item = []) {
  return Array.isArray(item[0]);
}

function parseIf (fieldMap, cond, context) {
  let expr = '';

  // field
  const fieldId = cond[0];
  let op = cond[1];
  let value = cond[2];
  const fieldInfo = fieldMap[fieldId];
  let fieldName = `${context.objName && (context.objName + '.') || ''}${fieldId}`;

  // op
  if (cond.length === 0) {
    return !expr ? '1 === 1' : expr;
  } else if (op === 'be_null') {
    expr = `(${fieldName} === '' || ${fieldName} === null || ${fieldName} === undefined || ${fieldName}?.length ===0)`;
  } else if (op === 'not_be_null') {
    expr = `((!Array.isArray(${fieldName}) && ${fieldName} !== '' && ${fieldName} !== null && ${fieldName} !== undefined ) || (Array.isArray(${fieldName}) && ${fieldName}.length > 0))`;
  } else if (fieldInfo.controlType === COMPONENT_TYPE.NUMBER && (
    op === 'between' || op === 'not_between')) {
    if (op === 'between') {
      expr = `(${fieldName} >= ${value[0]} && ${fieldName} <= ${value[1]})`;
    } else if (op === 'not_between') {
      expr = `(${fieldName} < ${value[0]} || ${fieldName} > ${value[1]})`;
    }
  } else if (fieldInfo.controlType === COMPONENT_TYPE.MEMBER) {
    expr += `${fieldName}?.length===${value.length}&&(`;
    const tempOp = '>=0';

    value.forEach((item, index) => {
      if (index !== 0) expr += ` && `;
      expr += `${fieldName}.map(item=>item.user_id).indexOf(${item.split('_')[0]})${tempOp}`;
    });
    expr += `)`;
    expr = op === '=' ? expr : ('!(' + expr + ')');
  } else if (Array.isArray(value) && (op === '=' || op === '!=')) {
    expr += `${fieldName}?.length===${value.length}&&(`;
    const tempOp = '>=0';

    value.forEach((item, index) => {
      if (index !== 0) expr += ` && `;
      expr += `${fieldName}.indexOf(${typeof item === 'string' ? `'${item}'` : item})${tempOp}`;
    });
    expr += `)`;
    expr = op === '=' ? expr : ('!(' + expr + ')');
  }
  else if ((fieldInfo.controlType === COMPONENT_TYPE.TEXT
    || fieldInfo.controlType === COMPONENT_TYPE.TEXTAREA
    || fieldInfo.controlType === COMPONENT_TYPE.EMAIL
    || fieldInfo.controlType === COMPONENT_TYPE.TELEPHONE) && (op === 'contain' || op === 'not_contain')) {
    expr = `(${fieldName} + '').indexOf('${(value || '').replace(/\'/g, '\\\'')}') ${op === 'contain' ? '>' : '==='} -1`;
  } else if (fieldInfo.controlType === COMPONENT_TYPE.DATE) {
    let date_length = 10;
    if (fieldInfo.picker === 'year') {
      date_length = 4;
    } else if (fieldInfo.picker === 'month') {
      date_length = 7;
    }
    fieldName = `(new Date(${fieldName})).toJSON()?.substr(0, ${date_length})`;
    value = `(new Date('${value || ''}')).toJSON()?.substr(0, ${date_length})`;

    switch (op) {
      case '=': op = '==='; break;
      case '<>': op = '!='; break;
      case 'before': op = '<'; break;
      case 'after': op = '>'; break;
    }
    expr = `${fieldName} ${op} ${value}`;
  } else {
    switch (op) {
      case '=': op = '==='; break;
      case '<>': op = '!='; break;
    }
    expr = `${fieldName} ${op} ${typeof value === 'string' ? `'${value}'` : value}`;
  }

  return !expr ? '1 === 1' : expr;
}

function parseConditionBase (fieldMap, conditions = [], isOr = true, context) {
  if (!isCondition(conditions)) {
    return parseIf(fieldMap, conditions, context);
  }
  const list = [];
  conditions.forEach(cond => {
    const result = parseConditionBase(fieldMap, cond, !isOr, context);
    list.push(result);
  });
  return '(' + list.join(isOr ? ' || ' : ' && ') + ')';
}

function parseCondition (form, conditions = [], isOr = true, context) {
  const fieldMap = {};
  form.fields.forEach(field => {
    fieldMap[field.id] = field;
  });
  let result = parseConditionBase(fieldMap, conditions, isOr, context);
  result = `return function (${context.objName || ''}) { return ${result}; }`;
  return (new Function(result))();
}

/**
 *
 * @param {*} form: object 表单配置
 * @param {*} conditionConfig: object 条件列表
 * @param {*} context = {
 *      item: { 任务信息
 *        owner: { id, name, type },  任务的负责人信息
 *        workers: [{ id, name, type }]  任务的参与人信息
 *      },
 *      currentUser: {  当前用户信息
 *        id, name,
 *        roles: [ { id, name, type } ] 当前用户在项目的角色信息
 *      }
 *              objName : 'item', // 对象名称，比如'item'、'obj'、'user'
 *            }  上下文信息，如当前用户的角色列表等
 * @returns function (obj) { return obj.a === 1 || obj.c > 2; }
 *
 * const fn =
 *
 * list.filter(fn)
 *
 * list.filter(item => return item.a==1 && item.b==2)
 */
function parseConditionConfig (form, conditionConfig = {}, context = { objName: '' }) {
  let result = '';
  const userOwnerTypes = calcUserOwnerTypes(context);
  let condition_filter_merges = [];
  // 合并不同角色的条件，互为||
  for (const userOwnerType of userOwnerTypes) {
    const cond = conditionConfig[userOwnerType];
    if (cond && cond.filters && cond.filters.length > 0 && cond.filters.length > 0) {
      condition_filter_merges = [...condition_filter_merges, ...cond.filters];
    }
    // if (cond?.filters?.length > 0 && cond?.filters[0]?.length > 0) {
    //   condition_filter_merges[0] = [...condition_filter_merges[0], ...cond.filters[0]];
    // }
  }
  result = parseCondition(form, condition_filter_merges, true, context);
  // console.info(result);
  return result;
}

module.exports = {
  parseConditionConfig,
  parseCondition,
};
