const { COMPONENT_TYPE } = require("../component/component_type")
// const helper = require('../helper');
const {
  Calculator,
  translate,
  parse,
  parseFormulaStaStructure,
} = require("../formula/lib/formula_main")
const helper = require("../helper")

function FormManager (
  cfg = {
  }
) {
  cfg = {
    onFieldValueChanged (rowIndex, fieldId, value) { },
    getFieldValue (rowIndex, fieldId, value) { },
    getCurrentDate () {
      return new Date().getTime()
    },
    async getNumber (fieldId, form_datas) {
      return null
    },
    async getFormInfo (tableUniqueNo) {
      return null
    },
    ...cfg,
  };
  let fieldMap = {},
    fieldNameMap = {}
  let dependMap = {},
    dependedMap = {}
  let form = {}
  let formDataList = []
  let newRowStartIndex = 0

  function getFieldDefaultValue (fid, value) {
    const fieldInfo = fieldMap[fid]
    if (!fieldInfo || (value !== null && value !== undefined)) return value
    // todo gefei
    switch (fieldInfo.controlType) {
      case COMPONENT_TYPE.AUTO_NUMBER:
        return ""
      case COMPONENT_TYPE.DATE:
        if (fieldInfo.dateDefaultValue === "create") {
          let data = cfg.getCurrentDate()
          return data
        }
    }

    return fieldInfo.value
  }

  async function getFieldSubmitValue (fid, value, data) {
    const fieldInfo = fieldMap[fid]
    if (!fieldInfo) return null
    if (
      value !== null &&
      value !== undefined &&
      fieldInfo.controlType !== COMPONENT_TYPE.AUTO_NUMBER
    )
      return value

    // todo gefei
    switch (fieldInfo.controlType) {
      case COMPONENT_TYPE.AUTO_NUMBER:
        const no = await cfg.getNumber(fid, data)
        if (no === null) return undefined
        // todo by xiaoji: 添加编号生成代码
        return no
      case COMPONENT_TYPE.DATE:
        return helper.Date.getDateByFormat(value, "yyyy/MM/dd HH:mm:ss")
    }

    return value
  }
  /**
   * 绑定原始数据，绑定的是从服务器拿到的原始数据
   * 给自动编号数据默认null
   * @param {*} data: object[], [{ f1: 1, f2: { id: 1, label: '男' } },...]
   */
  const bindData = function (dataList = []) {
    formDataList = JSON.parse(JSON.stringify(dataList))
    newRowStartIndex = formDataList.length

    const fields = Object.keys(fieldMap)
    formDataList.forEach((row, index) => {
      const formulas = [];
      fields.forEach((fid) => {
        const fieldInfo = fieldMap[fid];
        if (fieldInfo.controlType === COMPONENT_TYPE.MATH_FORMULA) {
          formulas.push(fieldInfo);
          return;
        }
        row[fid] =
          row[fid] === null || row[fid] === undefined
            ? getFieldDefaultValue(fid)
            : row[fid]
      })
      if (formulas.length > 0) {
        row = {
          ...row,
          ...model.calc(index, formulas[0].id, formulas),
        };
      }
    })
  }

  /**
   * 获取表数据，将从服务器拿到的数据，进行默认值、公式计算补齐后返回
   * @param {*} data
   * @returns
   */
  const getData = function () {
    return formDataList
  }

  /**
   * 获取表提交时的数据，即提交给服务器接口的数据格式
   * @param {*} data
   * @returns
   */
  const getSubmitData = async function () {
    const newRowList = []
    for (let i = 0; i < formDataList.length; i++) {
      const row = formDataList[i]
      const fids = Object.keys(row)
      const newRow = {}
      newRowList.push(newRow)

      for (let j = 0; j < fids.length; j++) {
        const fid = fids[j]
        newRow[fid] = await getFieldSubmitValue(fid, row[fid], row)
      }
    }
    return newRowList
  }

  function renderCalculator (rowData) {
    return new Calculator(
      getFormulaFieldInfo,
      function (fid) {
        const fieldInfo = fieldMap[fid]
        if (!fieldInfo) return null

        const value = rowData[fid]
        if (fieldInfo.controlType === COMPONENT_TYPE.AUTO_NUMBER) {
          if (typeof value === "object") {
            return value.value
          }
        }
        return value
      },
      function () {
        return 1
      },
      getFormulaFieldIdByName,
      getFormulaFieldNameById
    )
  }

  /**
   * 获取表新行数据，按照数据展示的格式
   * @param {*} data
   * @returns
   */
  const getNewRowData = function () {
    const fields = Object.keys(fieldMap)
    const row = {}
    fields.forEach((fid) => {
      const fieldInfo = fieldMap[fid]
      if (
        fieldInfo.controlType === COMPONENT_TYPE.AUTO_NUMBER ||
        fieldInfo.controlType === COMPONENT_TYPE.MATH_FORMULA
      )
        return
      row[fid] = getFieldDefaultValue(fid)
    })
    fields.forEach((fid) => {
      const fieldInfo = fieldMap[fid]
      if (
        fieldInfo.controlType !== COMPONENT_TYPE.MATH_FORMULA ||
        fieldInfo.__parseError
      )
        return

      const _calculator = renderCalculator(row)
      _calculator.addExpr(fid, fieldInfo.expr)
      _calculator.calc(function (fid, value) {
        row[fid] = value
      })
    })
    return row
  }

  /**
   * 获取表新行数据，按照提交保存的格式
   * @param {*} data
   * @returns
   */
  const getNewRowSubmitData = async function () {
    const row = getNewRowData()
    const fids = Object.keys(row)
    for (let j = 0; j < fids.length; j++) {
      const fid = fids[j]
      row[fid] = await getFieldSubmitValue(fid, row[fid], {})
    }
    return row
  }

  const addRowData = function (rowIndex, data) {
    if (formDataList[rowIndex]) return
    formDataList[rowIndex] = data
  }

  const removeRowData = function (rowIndexs = []) {
    for (let i = rowIndexs.length - 1; i >= 0; i--) {
      const rowIndex = rowIndexs[i]
      if (!formDataList[rowIndex])
        throw new Error(`not exists row: ${rowIndex}`)
      formDataList.splice(rowIndex, 1)
      if (rowIndex < newRowStartIndex) newRowStartIndex = newRowStartIndex - 1
    }
  }

  const setFieldValue = function (rowIndex, fieldData) {
    if (!formDataList[rowIndex]) throw `${rowIndex} not exists`
    let changedFields = {}

    Object.keys(fieldData).forEach((fid) => {
      const value = fieldData[fid]
      formDataList[rowIndex][fid] = value

      if (!dependMap[fid]) return
      changedFields[fid] = value
      Object.keys(dependMap[fid]).forEach((formulaFieldId) => {
        changedFields = {
          ...changedFields,
          ...model.calc(rowIndex, formulaFieldId),
        }
      })
    })
    return changedFields
  }

  function getFormulaFieldInfo (fid) {
    const fieldInfo = fieldMap[fid]
    return fieldInfo
      ? {
        id: fid,
        table: "main",
        name: fieldInfo.name,
        vtype:
          fieldInfo.controlType === COMPONENT_TYPE.MATH_FORMULA
            ? fieldInfo.vtype
            : fieldInfo.controlType == COMPONENT_TYPE.NUMBER
              ? "num"
              : fieldInfo.controlType == COMPONENT_TYPE.DATE
                ? "time"
                : "",
        isTableField: true,
      }
      : undefined
  }

  function getFormulaFieldNameById (fid) {
    const fieldInfo = fieldMap[fid]
    return fieldInfo ? fieldInfo.name : "已删？"
  }

  function getFormulaFieldIdByName (name) {
    const fieldInfos = fieldNameMap[name]
    return fieldInfos && fieldInfos.length === 1 ? fieldInfos[0].id : ""
  }

  var _translate = function (_expr) {
    return translate(_expr || "", getFormulaFieldInfo, getFormulaFieldNameById)
  }

  var _parse = function (_expr) {
    return parse(_expr || "", getFormulaFieldInfo, getFormulaFieldIdByName)
  }

  /**
   * 绑定主表信息
   * @param {*} sourceForm
   */
  function bindForm (sourceForm = {}) {
    form = JSON.parse(JSON.stringify(sourceForm))
    form.unique_no = `s` + form.unique_no
    parseFormInfo()
  }

  function checkField (fieldInfo) {
    // if (fieldMap[fieldInfo.id]) {
    //   throw new Error(`has repeat id: ${fieldInfo.id}, ${fieldInfo.name}`)
    // }

    // if (fieldNameMap[fieldInfo.name]) {
    //   throw new Error(`has repeat name: ${fieldInfo.id}, ${fieldInfo.name}`)
    // }
    return true
  }

  function renderCacheData (fieldInfo) {
    fieldInfo.name = fieldInfo.label
    fieldMap[fieldInfo.id] = fieldInfo
    fieldNameMap[fieldInfo.name] = fieldNameMap[fieldInfo.name] || []
    fieldNameMap[fieldInfo.name].push(fieldInfo)
  }

  function removeFromCacheData (fid) {
    const fieldInfo = fieldMap[fid]
    delete fieldMap[fid]

    const list = fieldNameMap[fieldInfo.name]
    const count = list.length
    for (let i = count - 1; i >= 0; i--) {
      if (list[i].id === fid) {
        list.splice(i, 1)
      }
    }

    if (list.length === 0) delete fieldNameMap[fieldInfo.name]

    if (fieldInfo.controlType !== COMPONENT_TYPE.MATH_FORMULA) return
    for (let i = 0; i < fieldInfo.depend.length; i++) {
      const tempMap = dependMap[fieldInfo.depend[i]]
      delete tempMap[fid]
    }
  }

  function parseFormInfo () {
    fieldMap = {}
    fieldNameMap = {}
    stringFormulaMap = {}

    const fields = form.setting ? form.setting.fields : []
    fields.forEach((item) => {
      item.name = item.label
      checkField(item)
      renderCacheData(item)
    })
    fields.forEach((item) => {
      renderField(item)
    })
  }

  function renderField (fieldInfo) {
    if (fieldInfo.controlType === COMPONENT_TYPE.MATH_FORMULA) {
      try {
        const r = _parse(_translate(fieldInfo.expr))
        if (r.err) {
          throw new Error(r.err)
        }
        const { vtype, table, depend, mathFunName, sexpr } = r
        fieldInfo.depend = depend
        fieldInfo.vtype = vtype
        if (depend && depend.length > 0) {
          depend.forEach((d) => {
            dependMap[d] = dependMap[d] || {}
            dependMap[d][fieldInfo.id] = 1
          })
        }
      } catch (ex) {
        console.warn("parse formula error", fieldInfo, ex)
        fieldInfo.__parseError = true
      }
    }
  }

  const model = {
    bindForm,
    updateField (fid, fieldConfig) {
      const old = fieldMap[fid]
      const oldName = old.name

      const item = JSON.parse(JSON.stringify(fieldConfig))
      item.name = item.label
      checkField(item)

      if (oldName !== fieldConfig.name) {
        const list = fieldNameMap[oldName]
        for (let i = list.length - 1; i >= 0; i--) {
          if (list[i].id === fid) {
            list.splice(i, 1)
          }
        }
        if (list.length === 0) delete fieldNameMap[oldName]
      }
      renderField(item)
    },
    addField (fieldConfig) {
      const item = JSON.parse(JSON.stringify(fieldConfig))
      checkField(item)
      renderField(item)
    },
    removeField (fid) {
      const fieldInfo = fieldMap[fid]
      if (dependMap[fid])
        throw `字段被${Object.keys(dependMap[fid])
          .map((fid) => fieldMap[fid].name)
          .join("、")}字段依赖，不能删除！`

      removeFromCacheData(fid)
    },
    parseExpr (expr) {
      const { vtype, table, depend, mathFunName, sexpr, err } = _parse(expr)
      return { sexpr, err }
    },
    translateExpr (fid) {
      const fieldInfo = fieldMap[fid]
      return _translate(fieldInfo.expr)
    },

    bindData,
    getData,
    getSubmitData,
    getNewRowData,
    getNewRowSubmitData,
    addRowData,
    removeRowData,
    setFieldValue,
    calc (rowIndex = 0, fid, exprs) {
      const fieldInfo = fieldMap[fid]
      let changedFields = {}

      if (fieldInfo.__parseError) return changedFields

      // for (let i = 0; i < fieldInfo.depend.length; i++) {
      //   if (
      //     (formDataList[rowIndex][fieldInfo.depend[i]] === null ||
      //       formDataList[rowIndex][fieldInfo.depend[i]] === undefined) &&
      //     fieldMap[fieldInfo.depend[i]].controlType ===
      //     COMPONENT_TYPE.MATH_FORMULA
      //   ) {
      //     const r = model.calc(rowIndex, fieldInfo.depend[i])
      //     changedFields = {
      //       ...changedFields,
      //       ...r,
      //     }
      //   }
      // }

      const _calculator = renderCalculator(formDataList[rowIndex])
      if (exprs) {
        exprs.forEach(item => {
          _calculator.addExpr(item.id, item.expr)
        });
      } else {
        Object.keys(fieldMap).forEach(fid => {
          if (fieldMap[fid].controlType === COMPONENT_TYPE.MATH_FORMULA) {
            _calculator.addExpr(fid, fieldMap[fid].expr)
          }
        });
      }
      _calculator.calc(function (fid, value) {
        formDataList[rowIndex][fid] = value
        changedFields[fid] = value
        // cfg.onFieldValueChanged(rowIndex, fid, value)
      })
      return changedFields
    },
  }

  return model
}

module.exports = {
  FormManager,
}
