/* 说明:
 grp: true表示是组统计函数, 没有或false表示是常规函数
 vtypes: 函数返回值以及参数类型数组, [0]是返回值类型, [1][2]...分别是第1,2,...个参数的类型
 code: js代码模板, $arg1$表示第一个参数 目前仅对组函数有效
 */
const funcs = {
  DATE_ADD: {
    title: "时间计算",
    isGroupFun: false,
    exprs: {
      "time,num,string": {
        vtype: "time",
        grp: false,
        fn(dateString = "", num = 0, type) {
          if (!dateString) return null
          const date = new Date(dateString)
          switch (type) {
            case "second":
              date.setSeconds(date.getSeconds() + num)
              break
            case "minute":
              date.setMinutes(date.getMinutes() + num)
              break
            case "hour":
              date.setHours(date.getHours() + num)
              break
            case "day":
              date.setDate(date.getDate() + num)
              break
            case "month":
              date.setMonth(date.getMonth() + num)
              break
            case "year":
              date.setFullYear(date.getFullYear() + num)
              break
          }
          return date
        },
      },
    },
  },
  SUM: {
    title: "合计",
    exprs: {
      num: {
        vtype: "num",
        grp: true,
        code:
          "var x=0,$i=0,xs=[];" +
          "for(;$i<$n;++$i)" +
          "xs.push($arg1$);" +
          "x=globalThis.Formula.floatcalculate.adds.apply(null,xs);" +
          "return x;",
      },
    },
  },
  AVG: {
    title: "平均值",
    exprs: {
      num: {
        vtype: "num",
        grp: true,
        code:
          "var x=0,$i=0,xs=[];" +
          "for(;$i<$n;++$i)" +
          "xs.push($arg1$);" +
          "x=globalThis.Formula.floatcalculate.adds.apply(null,xs);" +
          "return $i==0?0:globalThis.Formula.floatcalculate.div(x,$i);",
      },
    },
  },
  MAX: {
    title: "最大值",
    exprs: {
      num: {
        vtype: "num",
        grp: true,
        code: [
          "var n=undefined, $i=0;",
          "for(;$i<$n;$i++)",
          "n=n==undefined?$arg1$:Math.max(n, $arg1$);",
          "return n;",
        ].join(""),
      },
      time: {
        vtype: "time",
        grp: true,
        code: [
          "var n=undefined, $i=0;",
          "for(;$i<$n;$i++)if($arg1$)",
          "n=n==undefined?(new Date($arg1$)).valueOf():Math.max(n, (new Date($arg1$)).valueOf());",
          "return n!=null?new Date(n):'';",
        ].join(""),
      },
    },
  },
  MIN: {
    title: "最小值",
    exprs: {
      num: {
        vtype: "num",
        grp: true,
        code: [
          "var n=undefined, $i=0;",
          "for(;$i<$n;$i++)",
          "n=n==undefined?$arg1$:Math.min(n, $arg1$);",
          "return n;",
        ].join(""),
      },
      time: {
        vtype: "time",
        grp: true,
        code: [
          "var n=undefined, $i=0;",
          "for(;$i<$n;$i++)if($arg1$)",
          "n=n==undefined?(new Date($arg1$)).valueOf():Math.min(n, (new Date($arg1$)).valueOf());",
          "return n!=null?new Date(n):'';",
        ].join(""),
      },
    },
  },
  COUNT: {
    title: "计数",
    exprs: {
      num: {
        vtype: "num",
        grp: true,
        code: "return $n;",
      },
      string: {
        vtype: "num",
        grp: true,
        code: "return $n;",
      },
      time: {
        vtype: "num",
        grp: true,
        code: "return $n;",
      },
    },
  },
  IF: {
    title: "三目运算",
    exprs: {
      "num,num,num": {
        vtype: "num",
        grp: false,
        fn: function () {
          return arguments[0] ? arguments[1] : arguments[2]
        },
      },
      "num,time,time": {
        vtype: "time",
        grp: false,
        fn: function () {
          return arguments[0] ? arguments[1] : arguments[2]
        },
      },
      "num,string,string": {
        vtype: "string",
        grp: false,
        fn: function () {
          return arguments[0] ? arguments[1] : arguments[2]
        },
      },
    },
  },
  CEIL: {
    title: "向上取整",
    exprs: {
      num: {
        vtype: "num",
        grp: false,
        fn: function () {
          return Math.ceil(arguments[0])
        },
      },
    },
  },
  FLOOR: {
    title: "向下取整",
    exprs: {
      num: {
        vtype: "num",
        grp: false,
        fn: function () {
          return Math.floor(arguments[0])
        },
      },
    },
  },
  ROUND: {
    title: "四舍五入",
    exprs: {
      num: {
        vtype: "num",
        grp: false,
        fn: function () {
          return Math.round(arguments[0])
        },
      },
    },
  },
  NOT: {
    title: "非",
    exprs: {
      num: {
        vtype: "num",
        grp: false,
        fn: function () {
          return 1 - (arguments[0] ? 1 : 0)
        },
      },
      time: {
        vtype: "time",
        grp: false,
        fn: function () {
          return 1 - (arguments[0] ? 1 : 0)
        },
      },
    },
  },
  ANY: {
    title: "匹配至少一个存在",
    exprs: {
      num: {
        vtype: "num",
        grp: true,
        code: [
          "var n=0, $i=0;",
          "for(;$i<$n;++$i){",
          "if(!$arg1$){continue;}",
          "else{n=1;break;}",
          "}",
          "return n;",
        ].join(""),
      },
      time: {
        vtype: "time",
        grp: true,
        code: [
          "var n=0, $i=0;",
          "for(;$i<$n;++$i){",
          "if(!$arg1$){continue;}",
          "else{n=1;break;}",
          "}",
          "return n;",
        ].join(""),
      },
    },
  },
}

module.exports = {
  funcs,
  list: [
    {
      groupName: "日期函数",
      groupId: 1,
      options: [
        {
          title: "DATE_ADD",
          desc: "返回当前日期和指定日期之间的差值（无正负）",
          detail: [
            "【date】是你指定的日期。本函数将在该日期基础上增加一定的时间间隔",
            "【count】是时间间隔",
            "【units】是计时单位",
          ],
        },
      ],
    },
    {
      groupName: "数字函数",
      groupId: 2,
      options: [],
    },
  ],
}
