const helper = require('../../helper');
const StringFormula = function (cfg) {
    var config = Object.assign({
        getFieldInfo: function () { },
        getFieldData: function () { },
        getTableRowCount: function () { },
        getFieldInfoByName: function () { }
    }, cfg);


    var exprList = [], fieldStepInfo = {}, dependList = {}, exprs = {};

    var _parseExpr = function (_exprStr) {
        var _str = _exprStr, _reg = /\{[^\{\}]{1,}?\}/g, _matchArray = _str.match(_reg),
            len = _matchArray ? _matchArray.length : 0,
            cmatch, cfield, hasReplace = {}, _depends = {}, _html = _str, _extend = "<br/>",
            _table = "main", hasField = false, _tableInfo;
        for (var i = 0; i < len; i++) {
            cmatch = _matchArray[i];
            cfield = config.getFieldInfoByName(cmatch.replace(/\{|\}/g, ""));
            if (!cfield || hasReplace[cfield.id]) continue;
            hasReplace[cfield.id] = true;
            _tableInfo = cfield.table == "main" ? { name: "" } : config.getFieldInfo(cfield.table);
            if (cfield) {
                hasField = true;
                _depends[cfield.id] = 1;
                if (cfield.table != "main") _table = cfield.table;
                _str = _str.replace(new RegExp(cmatch, "g"), "{" + cfield.id + "}");
                _extend = cfield.controlType == "textarea" ? "<br/>" : "";
                _html = _html.replace(new RegExp(cmatch, "g"), _extend + "{" + (cfield.table == "main" ? "" : (_tableInfo.name + ".")) + cfield.name + "}" + _extend);
            }
        }

        return {
            expr: _str,
            str: _exprStr,
            depends: _depends,
            table: _table,
            html: _html,
            hasField: hasField
        };
    };
    var _translateExpr = function (_expr) {
        var _str = _expr,
            _reg = /\{[^\{\}]{2,}?\}/g,
            _matchArray = _str.match(_reg),
            len = _matchArray ? _matchArray.length : 0,
            cmatch,
            cfield,
            hasReplace = {},
            _id,
            _depends = {}, _html = _str, _extend = "<br/>",
            _table = "main", hasField = false, _tableInfo;
        for (var i = 0; i < len; i++) {
            cmatch = _matchArray[i];
            _id = cmatch.replace(/\{|\}/g, "");
            cfield = config.getFieldInfo(_id);
            if (!cfield || hasReplace[cfield.id]) continue;
            hasReplace[cfield.id] = true;
            _tableInfo = cfield.table == "main" ? { name: "" } : config.getFieldInfo(cfield.table);
            if (cfield) {
                hasField = true;
                if (cfield.table != "main") _table = cfield.table;
                _depends[cfield.id] = 1;
                _str = _str.replace(new RegExp(cmatch, "g"), "{" + (cfield.table == "main" ? "" : (_tableInfo.name + ".")) + cfield.name + "}");
                _extend = cfield.controlType == "textarea" ? "<br/>" : "";
                _html = _html.replace(new RegExp(cmatch, "g"), _extend + "{" + (cfield.table == "main" ? "" : (_tableInfo.name + ".")) + cfield.name + "}" + _extend);
            }
        }

        return {
            expr: _expr,
            str: _str,
            depends: _depends,
            table: _table,
            hasField: hasField,
            html: _html
        };
    };

    var getValueByControlType = function (_fieldInfo, _data, noHtml) {
        _data = _data == null ? "" : _data;
        switch (_fieldInfo.controlType) {
            case "weaksignature":
            case "user": return _data ? _data.name || _data : "";
            case "orgunit": return _data ? _data.path || _data.name || _data : "";
            case "date": return _data == "" ? "" : helper.Date.getDateByFormat(_data, _fieldInfo.dateFormat) || "";
            case "textarea": return _data ? noHtml ? _data : ("<br/>" + _data + "<br/>") : "";
            default: return _data == null ? "" : _data;
        }

    };
    var _translateByData = function (_expr, _getFieldData) {
        if (!_expr) return "";
        var _matchArray = _expr.match(/\{[^\{\}]{2,}?\}/g),
            len = _matchArray ? _matchArray.length : 0,
            cmatch, cfield, _id, _str = _expr;

        for (var index = 0; index < len; index++) {
            cmatch = _matchArray[index];
            _id = cmatch.replace(/\{|\}/g, "");
            cfield = config.getFieldInfo(_id);
            if (cfield) {
                _str = _str.replace(new RegExp(cmatch, "g"), getValueByControlType(cfield, _getFieldData(cfield.id)));
            }
        }

        return _str;
    };

    var findWhoDepend = function (_fieldId) {

        var _fields = [];

        for (var e in exprs) {
            if (findSameDepend(exprs[e].expr.depends, _fieldId)) _fields.push(e);
        }

        return _fields;
    };

    var findSameDepend = function (_whos, _dependWho) {
        var isSame = false;
        if (_whos[_dependWho]) return true;
        for (var i in _whos) {
            var ciDepends = exprs[i] ? exprs[i].expr.depends : null;
            if (!ciDepends) continue;
            isSame = findSameDepend(ciDepends, _dependWho);
            if (isSame) break;
        }

        return isSame;
    };
    var _addExpr = function (_fieldId, _expr) {
        var _item = { expr: _translateExpr(_expr), id: _fieldId }, i, ci, minIndex, _depend;

        if (findSameDepend(_item.expr.depends, _fieldId)) return;
        exprs[_fieldId] = _item;

        for (_depend in _item.expr.depends) {
            dependList[_depend] = dependList[_depend] || {};
            dependList[_depend][_fieldId] = 1;
        }

        if (dependList[_fieldId]) {
            for (_depend in dependList[_fieldId]) {
                minIndex = minIndex == undefined ? fieldStepInfo[_depend] : Math.min(minIndex, fieldStepInfo[_depend]);
            }
        }

        if (minIndex == undefined) {
            exprList.push(_fieldId);
            fieldStepInfo[_fieldId] = exprList.length - 1;
            return;
        }
        var _lasts = exprList.splice(minIndex);
        exprList.push(_fieldId);
        fieldStepInfo[_fieldId] = exprList.length - 1;

        for (i = 0; i < _lasts.length; i++) {
            ci = _lasts[i];
            exprList.push(ci);
            fieldStepInfo[ci] += 1;
        }
    };

    var _translateExprs = function (_onChange, _getFieldData, _getRowCount, _exprList) {
        var ci, ce, rowLen;
        _exprList = _exprList || exprList;
        for (var i = 0, j = _exprList.length; i < j; i++) {
            ci = _exprList[i];
            if (!ci) continue;
            ce = exprs[ci];
            if (!ce) continue;
            if (ce.expr.table == "main") {
                _onChange(ci, _translateByData(ce.expr.expr, config.getFieldData), ce.expr.table);
            } else {
                rowLen = config.getTableRowCount(ce.expr.table) || 0;
                for (var m = 0; m < rowLen; m++) {
                    _onChange(ci, _translateByData(ce.expr.expr, function (_f) {
                        return config.getFieldData(_f, m);
                    }), ce.expr.table, m);
                }
            }
        }
    };

    var _model = {
        translate: _translateExpr,
        parse: _parseExpr,
        calcExprs: _translateExprs,
        clear: function () {
            exprList = [];
            fieldStepInfo = {};
            dependList = {};
            exprs = {};
        },
        reCalc: function () { },
        addExpr: _addExpr,
        findSameDepend: findSameDepend,
        findWhoDepend: findWhoDepend,
        translateWithData: _translateByData
    };
    return _model;
};

module.exports = StringFormula;
