
const { FormFieldsManager } = require('../src/lib/form/form_field');
const StringFormula = require('../src/lib/formula/lib/formula_string');
const tableInfo = require('./data/table.json');
const tableInfoStr = JSON.stringify(tableInfo);
const MathFormulaManager = require('../src/lib/formula/math');



const formFieldsManager = new FormFieldsManager();

formFieldsManager.bindFields(tableInfo);

const _formStringFormula = new StringFormula({
  getFieldInfo: formFieldsManager.getFormulaFieldInfo,
  getFieldInfoByName: formFieldsManager.getFormulaFieldInfoByName,
  getFieldData: null,
  // getTableRowCount: config.getTableRowCount
});
 // 用户编辑的
const str = '{归属部门} {费用归属部门} {报销人} {报销日期}';

let result = _formStringFormula.parse(str);
// 数据库记录的：'{f3} {f4} {f1} {f2}'
console.info(result.expr);

// 根据数据解析公式：'信息开发中心 开发部 周维明 2021年01月01日'
console.info(_formStringFormula.translateWithData(result.expr, function (fieldId) {
  switch (fieldId) {
    case 'f3': return '信息开发中心';
    case 'f4': return '开发部';
    case 'f1': return '周维明';
    case 'f2': return '2021/01/01';
  }
}));

// console.info(result);
const mathFormulaManager = new MathFormulaManager({
  fieldsManager: formFieldsManager,
  getFieldData: function (fieldId) {
    return 10;
  },
  getTableRowCount: function () {
    return 10;
  },
  onCalcChange: function (fieldId, value) {
    console.info(fieldId, value);
  },
  validate: function () { return { retCode: 1 }; }
});
mathFormulaManager.bindFields(JSON.parse(tableInfoStr));

  // 's1.f24': "(FLOOR((s1.f14-s1.f13)*2)+CEIL(s1.f14-s1.f13)-FLOOR(s1.f14-s1.f13))/2*(s1.f12.s1000015.f2.s1000014.f2)",
  // 'f39': "f1.s3.f2",


// 用户编辑的
const exprStrs = {
  's1.f14': '报销明细.起始时间',
  'f25': 'SUM(报销明细.差补)',
  'f28': 'SUM(IF(报销明细.费用类型="日常",报销明细.申报金额,0))',
  'f29': 'SUM(IF(报销明细.费用类型="差旅",报销明细.申报金额,0))',
  'f30': 'SUM(IF(报销明细.费用类型="专项费用",报销明细.申报金额,0))',
};

// 数据库记录的：
const exprs = {
  's1.f14': 's1.f13',
  'f25': "SUM(s1.f24)",
  'f28': "SUM(IF(s1.f17=\"1\",s1.f19,0))",
  'f29': "SUM(IF(s1.f17=\"2\",s1.f19,0))",
  'f30': "SUM(IF(s1.f17=\"4\",s1.f19,0))",
  'f31': "COUNT(s1.f19)",
};

// 根据数据计算结果：
const results = {
  's1.f14': '2020-01-01 00:00:00',
  'f25': 2234,
  'f28': 120,
  'f29': 320,
  'f30': 420,
};

Object.keys(exprs).forEach(fieldId => {
  console.info(fieldId);
  mathFormulaManager.addExpr(fieldId, exprs[fieldId]);
  const str = mathFormulaManager.translate(exprs[fieldId]);

  const expr = mathFormulaManager.parse(str);
  console.info(expr);
});

mathFormulaManager.calc();
