const {
  CHILD_TABLE,
  FOREIGN_REF,
  NUMBER,
  TEXT,
  MATH_FORMULA,
  DATE,
} = require("../src/lib/base/component_type").COMPONENT_TYPE

const PROJECT = {
  key: "sproject", // 表名
  table: "projects", // 实际表
  label: "项目", // 表名
  level: 1, // 层级，是否外层显示
  setting: {
    fields: [
      // 基础信息
      {
        id: "fid", // 别名
        label: "项目ID", // 名称
        shown: false, // 是否展示
        controlType: NUMBER,
      },
      {
        id: "fseq_no",
        label: "项目编号",
        shown: true,
        controlType: TEXT,
        condition: true,
      },
      // {
      //   id: 'fsponsor',
      //   label: '申办方',
      //   shown: true,
      //   controlType: TEXT,
      // },
      // {
      //   id: 'findication_comment',
      //   label: '适应症',
      //   shown: true,
      //   controlType: TEXT,
      // },
      // {
      //   id: 'fproduct_name',
      //   label: '产品名称',
      //   shown: true,
      //   controlType: TEXT,
      // },
      // {
      //   id: 'fproject_type',
      //   label: '项目类型',
      //   shown: true,
      //   controlType: TEXT,
      // },
      // {
      //   id: 'fproject_phase',
      //   label: '项目分期',
      //   shown: true,
      //   controlType: TEXT,
      // },
      // {
      //   id: 'fplan_inclusion',
      //   label: '计划入组数',
      //   shown: true,
      //   controlType: NUMBER,
      // },
      // {
      //   id: 'fplan_begin_date',
      //   label: '计划开始时间',
      //   shown: true,
      //   controlType: DATE,
      // },
      // {
      //   id: 'fplan_end_date',
      //   label: '计划结束时间',
      //   shown: true,
      //   controlType: DATE,
      // },
      // {
      //   id: 'fcurrent_project_milestone_id',
      //   source: 'smilestone',
      //   label: '当前里程碑',
      //   shown: true,
      //   controlType: FOREIGN_REF,
      // },
    ],
  },
  parse_info: {
    sql: `
      select id,
        company_id,
        seq_no,
        current_project_milestone_id,
        0 as is_deleted
      from projects
      where parent_id = -1
        and is_deleted = 0
        and company_id = :current_company_id
      union
      select -2, 1, '管理工时', null, 0
      union
      select -6, 1, '研发工时', null, 0
    `,
  },
}

const PROJECT_MEMBER = {
  key: "sprojectmember",
  table: "project_members",
  label: "项目成员",
  setting: {
    fields: [
      {
        id: "fid",
        label: "项目成员ID",
        shown: false,
        controlType: NUMBER,
      },
      {
        id: "fproject_id",
        source: "sproject",
        label: "项目",
        shown: true,
        controlType: FOREIGN_REF,
      },
      {
        id: "fuser_id",
        source: "suser",
        label: "用户",
        shown: true,
        controlType: FOREIGN_REF,
      },
    ],
  },
}

const PROJECT_MEMBER_ROLE = {
  key: "sprojectmemberrole",
  table: "project_member_roles",
  label: "项目成员角色",
  setting: {
    fields: [
      {
        id: "fid",
        label: "项目成员角色关联ID",
        shown: false,
        controlType: NUMBER,
      },
      {
        id: "fproject_member_id",
        source: "sprojectmember",
        label: "项目成员",
        shown: true,
        controlType: FOREIGN_REF,
      },
      {
        id: "fproject_role_id",
        source: "srole",
        label: "角色",
        shown: true,
        controlType: FOREIGN_REF,
      },
    ],
  },
  parse_info: {
    sql: `
    select
      project_member_role_relationships.id,
      project_member_role_relationships.company_id,
      project_members.id as project_member_id,
      case
        when project_member_role_relationships.role_type = 1
          then project_member_role_relationships.project_role_id
        else concat('s', cast(project_member_role_relationships.project_role_id as char))
      end as project_role_id,
      project_member_role_relationships.is_deleted
    from project_member_role_relationships
      left join project_members
        on project_members.id = project_member_role_relationships.project_member_id
          and project_members.company_id = project_member_role_relationships.company_id
    where project_member_role_relationships.is_deleted = 0
      and project_members.is_deleted = 0
      and project_member_role_relationships.company_id = :current_company_id
  `,
  },
}

const ROLE = {
  key: "srole",
  table: "roles",
  label: "项目角色",
  level: 1,
  setting: {
    fields: [
      {
        id: "fid",
        label: "角色ID",
        shown: false,
        controlType: NUMBER,
      },
      {
        id: "fname",
        label: "角色名称",
        shown: true,
        controlType: TEXT,
        condition: true,
      },
      {
        id: "fproject_id", // f + 本表字段
        source: "sproject", // 关联表
        label: "所属项目",
        shown: true,
        controlType: FOREIGN_REF,
      },
    ],
  },
  parse_info: {
    sql: `
    select concat('s', cast(roles.id as char)) as id,
      roles.company_id,
      roles.name,
      projects.id as project_id,
      roles.is_deleted,
      roles.code as unique_no
    from projects
      left join roles
        on roles.company_id = projects.company_id
    where projects.is_deleted = 0
      and projects.company_id = :current_company_id
      and roles.is_deleted = 0
      and roles.role_type = 2
      and roles.parent_id != -1
      and projects.company_id = :current_company_id
  `,
  },
}

const MILESTONE = {
  key: "smilestone",
  table: "project_milestones",
  label: "里程碑",
  setting: {
    fields: [
      {
        id: "fid",
        label: "里程碑ID",
        shown: false,
        controlType: NUMBER,
      },
      {
        id: "fproject_id",
        label: "项目",
        shown: true,
        source: "sproject",
        controlType: FOREIGN_REF,
        condition: false,
      },
      {
        id: "fname",
        label: "里程碑名称",
        shown: true,
        controlType: TEXT,
        condition: true,
      },
      // {
      //   id: 'fstaus',
      //   label: '里程碑状态',
      //   shown: true,
      //   controlType: NUMBER,
      // },
      // {
      //   id: 'factual_begin_at',
      //   label: '实际开始',
      //   shown: true,
      //   controlType: DATE,
      // },
      // {
      //   id: 'factual_end_at',
      //   label: '实际结束',
      //   shown: true,
      //   controlType: DATE,
      // },
      // {
      //   id: 'fplan_time_limit',
      //   label: '标准时限',
      //   shown: true,
      //   controlType: NUMBER,
      // },
    ],
  },
  parse_info: {
    sql: `
    select
      project_milestones.company_id,
      project_milestones.project_id,
      project_milestones.id,
      project_tasks.name,
      0 as is_deleted
    from project_milestones
      left join project_tasks
        on project_tasks.id = project_milestones.project_task_id
          and project_milestones.project_id = project_tasks.project_id
          and project_tasks.company_id = project_milestones.company_id
      left join projects
        on projects.is_deleted = 0
          and projects.company_id = project_milestones.company_id
          and projects.id = project_milestones.project_id
    where project_milestones.company_id = :current_company_id
      and project_milestones.is_deleted = 0
      and project_tasks.is_deleted = 0
      and projects.parent_id = -1
  `, // 查询里程碑信息
  },
}

const TASK = {
  key: "stask",
  table: "project_tasks",
  label: "任务",
  setting: {
    fields: [
      {
        id: "fid",
        label: "任务ID",
        shown: false,
        controlType: NUMBER,
      },
      {
        id: "fname",
        label: "任务名称",
        shown: true,
        controlType: TEXT,
        condition: true,
      },
      // {
      //   id: 'fstatus',
      //   label: '任务状态',
      //   shown: true,
      //   controlType: NUMBER,
      // },
      // {
      //   id: 'fstarted_at',
      //   label: '计划开始',
      //   shown: true,
      //   controlType: DATE,
      // },
      // {
      //   id: 'fended_at',
      //   label: '计划结束',
      //   shown: 'true',
      //   controlType: DATE,
      // },
      // {
      //   id: 'factual_started_at',
      //   label: '实际开始',
      //   shown: true,
      //   controlType: DATE,
      // },
      // {
      //   id: 'factual_ended_at',
      //   label: '实际结束',
      //   shown: true,
      //   controlType: DATE,
      // },
      {
        id: "fstandard_time",
        label: "标准工时",
        shown: true,
        controlType: NUMBER,
      },
      {
        id: "fproject_id",
        source: "sproject",
        label: "所属项目",
        shown: true,
        controlType: FOREIGN_REF,
      },
      {
        id: "fchild_project_id",
        source: "schildproject",
        label: "所属中心",
        shown: true,
        controlType: FOREIGN_REF,
      },
    ],
  },
  parse_info: {
    // sql: `
    // select
    //   project_tasks.id,
    //   project_tasks.company_id,
    //   project_tasks.project_id,
    //   -1 as child_project_id,
    //   project_tasks.name,
    //   project_tasks.is_deleted,
    //   project_tasks.standard_time
    // from project_tasks
    //   LEFT JOIN projects
    //     ON projects.id = project_tasks.project_id
    //         AND projects.company_id = project_tasks.company_id
    // WHERE project_tasks.is_deleted = 0
    //         AND projects.is_deleted = 0
    //         AND projects.parent_id = -1
    //         and project_tasks.company_id = :current_company_id
    // union
    // select
    //   project_tasks.id,
    //   project_tasks.company_id,
    //   parent_projects.id as project_id,
    //   projects.id as child_project_id,
    //   project_tasks.name,
    //   project_tasks.is_deleted,
    //   project_tasks.standard_time
    // from project_tasks
    //   left join projects
    //     on projects.id = project_tasks.project_id
    //       and projects.company_id = project_tasks.company_id
    //   left join projects as parent_projects
    //     on parent_projects.parent_id = -1
    //       and parent_projects.id = projects.parent_id
    //       and projects.company_id = parent_projects.company_id
    // where project_tasks.is_deleted = 0
    //   and project_tasks.company_id = :current_company_id
    //   and projects.is_deleted = 0
    //   and projects.parent_id != -1
    //   and parent_projects.is_deleted = 0
    // union
    // select
    //   project_tasks.id,
    //   project_tasks.company_id,
    //   -1 * project_tasks.type as project_id,
    //   -1 as child_project_id,
    //   project_tasks.name,
    //   project_working_times.is_deleted,
    //   0 as standard_time
    // from project_working_times
    //   left join project_tasks
    //     on project_tasks.id = project_working_times.project_task_id
    //       and project_tasks.company_id = project_working_times.company_id
    // where project_working_times.is_deleted = 0
    //   and project_working_times.is_actived = 1
    //   and project_working_times.company_id = :current_company_id
    //   and project_tasks.type in (2, 6)
    //   and project_tasks.is_deleted = 0
    // `, // todo 清洗child_project_id， 添加-2 -6 表示 管理、研发工时
  },
}

const TASK_USER = {
  key: "staskuser",
  table: "task_users",
  label: "任务负责人",
  setting: {
    fields: [
      {
        id: "fid",
        label: "关联ID",
        shown: false,
        controlType: NUMBER,
      },
      {
        id: "fproject_task_id",
        source: "stask",
        label: "任务",
        shown: false,
        controlType: FOREIGN_REF,
      },
      {
        id: "fuser_id",
        source: "suser",
        label: "人员",
        shown: false,
        controlType: FOREIGN_REF,
      },
    ],
  },
  // todo
  parse_info: {
    sql: `
      select
        ROW_NUMBER() over(order by b.project_task_id) as id,
        b.*
      from (
        select distinct a.* from (
      (
        select
          project_task_owner_relationships.company_id,
          project_task_owner_relationships.project_task_id,
          users.id as user_id,
          project_task_owner_relationships.is_deleted
        from project_task_owner_relationships
          left join users
            on users.id = project_task_owner_relationships.owner_id
              and project_task_owner_relationships.type in (1, 4)
              and users.company_id = project_task_owner_relationships.company_id
        where project_task_owner_relationships.company_id = :current_company_id
          and project_task_owner_relationships.is_deleted = 0
          and users.is_deleted = 0
      ) union (
        select
          a.company_id,
          a.project_task_id,
          c.user_id,
          a.is_deleted
        from (
          select
            project_task_owner_relationships.project_task_id,
            project_tasks.project_id,
            project_task_owner_relationships.owner_id,
            project_task_owner_relationships.company_id,
            project_task_owner_relationships.is_deleted
          from project_task_owner_relationships
            left join project_tasks on project_task_owner_relationships.company_id = project_tasks.company_id
              and project_task_owner_relationships.project_task_id = project_tasks.id and project_tasks.is_deleted = 0 and project_tasks.project_id > 0
          where project_task_owner_relationships.company_id = :current_company_id
            and project_task_owner_relationships.is_deleted = 0
            and project_task_owner_relationships.type in (2, 5)
            and project_tasks.id is not null
        ) as a
        left join (
          select
            pm.user_id,
            pm.project_id,
            b.project_role_id,
            b.role_type
          from project_members as pm
          join projects as p on pm.company_id = p.company_id and pm.project_id = p.id and p.is_deleted = 0 and p.parent_id > 0
          join (
            select
              pmrr.project_role_id,
              pmrr.role_type,
              pm.user_id,
              pm.project_id
            from project_member_role_relationships as pmrr
              left join project_members as pm on pmrr.company_id = pm.company_id and pmrr.project_member_id = pm.id and pm.is_deleted = 0
            where pmrr.company_id = :current_company_id and pmrr.is_deleted = 0
              and pm.user_id is not null
          ) as b on pm.user_id = b.user_id and p.parent_id = b.project_id
            union
          select
            pm.user_id,
            pm.project_id,
            pmrr.project_role_id,
            pmrr.role_type
          from project_member_role_relationships as pmrr
            left join project_members as pm on pmrr.company_id = pm.company_id and pmrr.project_member_id = pm.id and pm.is_deleted = 0
          where pmrr.company_id = :current_company_id and pmrr.is_deleted = 0 and pm.user_id is not null
        ) as c on a.owner_id = c.project_role_id and a.project_id = c.project_id and c.role_type = 1
        where c.user_id is not null
      ) union (
        select
          a.company_id,
          a.project_task_id,
          c.user_id,
          a.is_deleted
        from (
          select
            project_task_owner_relationships.project_task_id,
            project_tasks.project_id,
            project_task_owner_relationships.owner_id,
            project_task_owner_relationships.company_id,
            project_task_owner_relationships.is_deleted
          from project_task_owner_relationships
            left join project_tasks on project_task_owner_relationships.company_id = project_tasks.company_id
              and project_task_owner_relationships.project_task_id = project_tasks.id and project_tasks.is_deleted = 0 and project_tasks.project_id > 0
          where project_task_owner_relationships.company_id = :current_company_id
            and project_task_owner_relationships.is_deleted = 0
            and project_task_owner_relationships.type in (3, 6)
            and project_tasks.id is not null
        ) as a
        left join (
          select
            pm.user_id,
            pm.project_id,
            b.project_role_id,
            b.role_type
          from project_members as pm
          join projects as p on pm.company_id = p.company_id and pm.project_id = p.id and p.is_deleted = 0 and p.parent_id > 0
          join (
            select
              pmrr.project_role_id,
              pmrr.role_type,
              pm.user_id,
              pm.project_id
            from project_member_role_relationships as pmrr
              left join project_members as pm on pmrr.company_id = pm.company_id and pmrr.project_member_id = pm.id and pm.is_deleted = 0
            where pmrr.company_id = :current_company_id and pmrr.is_deleted = 0
              and pm.user_id is not null
          ) as b on pm.user_id = b.user_id and p.parent_id = b.project_id
            union
          select
            pm.user_id,
            pm.project_id,
            pmrr.project_role_id,
            pmrr.role_type
          from project_member_role_relationships as pmrr
            left join project_members as pm on pmrr.company_id = pm.company_id and pmrr.project_member_id = pm.id and pm.is_deleted = 0
          where pmrr.company_id = :current_company_id and pmrr.is_deleted = 0 and pm.user_id is not null
        ) as c on a.owner_id = c.project_role_id and a.project_id = c.project_id and c.role_type = 2
        where c.user_id is not null
      )
      ) as a
      ) as b
  `,
  },
}

const HOUR = {
  key: "shour",
  table: "project_working_times",
  label: "工时",
  level: 1,
  setting: {
    fields: [
      {
        id: "fid",
        label: "工时ID",
        shown: false,
        controlType: NUMBER,
      },
      {
        id: "fremark",
        label: "工时描述",
        shown: true,
        controlType: TEXT,
      },
      {
        id: "fproject_plan_time",
        label: "计划工时",
        shown: true,
        controlType: NUMBER,
      },
      {
        id: "factual_time",
        label: "实际工时",
        shown: true,
        controlType: NUMBER,
      },
      {
        id: "fbegin_date",
        label: "工时日期",
        shown: true,
        controlType: DATE,
      },
      {
        id: "fuser_id",
        source: "suser",
        label: "人员",
        shown: true,
        controlType: FOREIGN_REF,
      },
      {
        id: "fproject_task_id",
        source: "stask",
        label: "任务",
        shown: true,
        controlType: FOREIGN_REF,
      },
    ],
  },
  parse_info: {
    sql: `
    select * from project_working_times
    where is_deleted = 0 and is_actived = 1
      and company_id = :current_company_id
  `,
  },
}

const USER = {
  key: "suser",
  table: "users",
  label: "人员",
  level: 1,
  setting: {
    fields: [
      {
        id: "fid",
        label: "人员ID",
        shown: false,
        controlType: NUMBER,
      },
      {
        id: "fname",
        label: "人员姓名",
        shown: true,
        controlType: TEXT,
        condition: true,
      },
    ],
  },
  parse_info: {
    sql: `
    select *
    from users
    where users.type = 1
      and is_deleted = 0
      and company_id = :current_company_id
  `,
  },
}

const CHILD_PROJECT = {
  key: "schildproject",
  table: "projects",
  label: "中心",
  level: 1,
  setting: {
    fields: [
      {
        id: "fid",
        label: "项目ID",
        shown: false,
        controlType: NUMBER,
      },
      {
        id: "fname",
        label: "中心名称",
        shown: true,
        controlType: TEXT,
        condition: true,
      },
      {
        id: "fhospital_number",
        label: "中心编号",
        shown: true,
        controlType: TEXT,
      },
      {
        id: "fcurrent_project_milestone_id",
        source: "schildprojectmilestone",
        label: "当前里程碑",
        shown: true,
        controlType: FOREIGN_REF,
      },
      {
        id: "fparent_id",
        source: "sproject",
        label: "主项目",
        shown: true,
        controlType: FOREIGN_REF,
      },
    ],
  },
  parse_info: {
    sql: `
    select
      projects.id,
      projects.name,
      projects.parent_id,
      project_child_level_extends.hospital_number,
      projects.is_deleted,
      projects.company_id
    from projects
      left join projects as parent_projects
        on projects.parent_id = parent_projects.id
          and parent_projects.company_id = projects.company_id
      left join project_child_level_extends
        on project_child_level_extends.project_id = projects.id
          and project_child_level_extends.company_id = projects.company_id
    where projects.parent_id != -1
      and projects.company_id = :current_company_id
      and parent_projects.is_deleted = 0
      and projects.is_deleted = 0
  `,
  },
}

const CHILD_PROJECT_MILESTONE = {
  key: "schildprojectmilestone",
  table: "project_milestones",
  label: "里程碑",
  setting: {
    fields: [
      {
        id: "fid",
        label: "里程碑ID",
        shown: false,
        controlType: NUMBER,
      },
      {
        id: "fchild_project_id",
        label: "中心",
        shown: true,
        source: "schildproject",
        controlType: FOREIGN_REF,
        condition: false,
      },
      {
        id: "fname",
        label: "里程碑名称",
        shown: true,
        controlType: TEXT,
        condition: true,
      },
      // {
      //   id: 'fstaus',
      //   label: '里程碑状态',
      //   shown: true,
      //   controlType: NUMBER,
      // },
      // {
      //   id: 'factual_begin_at',
      //   label: '实际开始',
      //   shown: true,
      //   controlType: DATE,
      // },
      // {
      //   id: 'factual_end_at',
      //   label: '实际结束',
      //   shown: true,
      //   controlType: DATE,
      // },
      // {
      //   id: 'fplan_time_limit',
      //   label: '标准时限',
      //   shown: true,
      //   controlType: NUMBER,
      // },
    ],
  },
  parse_info: {
    sql: `
    select
      project_milestones.company_id,
      project_milestones.project_id as child_project_id,
      project_milestones.id,
      project_tasks.name,
      0 as is_deleted
    from project_milestones
      left join project_tasks
        on project_tasks.id = project_milestones.project_task_id
          and project_milestones.project_id = project_tasks.project_id
          and project_tasks.company_id = project_milestones.company_id
      left join projects
        on projects.is_deleted = 0
          and projects.company_id = project_milestones.company_id
          and projects.id = project_milestones.project_id
    where project_milestones.company_id = :current_company_id
      and project_milestones.is_deleted = 0
      and project_tasks.is_deleted = 0
      and projects.parent_id != -1
  `, // 查询里程碑信息
  },
}

const DEPARTMENT = {
  key: "sdepartment",
  table: "departments",
  label: "部门",
  level: 1,
  setting: {
    fields: [
      {
        id: "fid",
        label: "部门ID",
        shown: false,
        controlType: NUMBER,
      },
      {
        id: "fname",
        label: "部门",
        shown: true,
        controlType: TEXT,
        condition: true,
      },
      {
        id: "fparent_id",
        source: "sdepartment",
        label: "上级部门",
        shown: true,
        controlType: FOREIGN_REF,
      },
    ],
  },
}

const DEPARTMENT_USER = {
  key: "sdepartmentuser",
  table: "department_user_relationships",
  label: "部门人员",
  setting: {
    fields: [
      {
        id: "fid",
        label: "部门人员关联ID",
        shown: false,
        controlType: NUMBER,
      },
      {
        id: "fdepartment_id",
        source: "sdepartment",
        label: "部门",
        shown: true,
        controlType: FOREIGN_REF,
      },
      {
        id: "fuser_id",
        source: "suser",
        label: "人员",
        shown: true,
        controlType: FOREIGN_REF,
      },
    ],
  },
}

const TABLE_MAP = {
  PROJECT,
  USER,
  DEPARTMENT,
  ROLE,
  HOUR,
  CHILD_PROJECT,
  CHILD_PROJECT_MILESTONE,
  PROJECT_MEMBER,
  MILESTONE,
  TASK,
  TASK_USER,
  PROJECT_MEMBER_ROLE,
  DEPARTMENT_USER,
}

function parseTables(tables = []) {
  tables = JSON.parse(JSON.stringify(tables))
  // console.info('==========parseTables table', tables);
  const tableMap = {}
  const childMap = {}
  const tableNameMap = {}
  const result = {}
  const GLOBAL_TABLE = JSON.parse(JSON.stringify(TABLE_MAP))
  Object.keys(GLOBAL_TABLE).forEach((key) => {
    const item = GLOBAL_TABLE[key]
    const tableKey = item.key
    tableMap[tableKey] = item
    result[key] = item
    tableNameMap[item.label] = item
    item.isSystemTable = true
    item.setting.fieldMap = {}
    item.setting.foreignRefMap = {}
    item.name = item.label
    let index = item.setting.fields.length
    item.setting.fields.forEach((field) => {
      item.setting.fieldMap[field.id] = field
      field.vtype =
        field.controlType === NUMBER
          ? "num"
          : field.controlType === DATE
          ? "time"
          : ""

      if (field.controlType === FOREIGN_REF) {
        item.setting.foreignRefMap[field.source] = field
        childMap[field.source] = childMap[field.source] || {}
        childMap[field.source][tableKey] = {
          label: item.label,
          name: item.name,
          shown: true,
          source: tableKey,
          controlType: CHILD_TABLE,
        }
      }
    })
  })

  tables.forEach((item) => {
    const tableKey = item.key
    tableMap[tableKey] = item
    item.table = tableKey
    item.label = item.name || item.label
    tableNameMap[item.label] = item

    let maxNum = 0
    item.setting.fields.forEach((field) => {
      if (/^\d+$/g.test(field.id.replace("f", ""))) {
        maxNum = Math.max(maxNum, Number(field.id.replace("f", "")))
      }
    })

    let foreignMap = {}
    item.setting.fieldMap = {}
    item.setting.fieldNameMap = {}
    item.setting.foreignRefMap = {}
    for (let i = 0; i < item.setting.fields.length; i++) {
      const field = item.setting.fields[i]
      if (
        field.id !== "fid" &&
        field.controlType !== FOREIGN_REF &&
        field.controlType !== MATH_FORMULA
      ) {
        const expr = field.expr

        const exprArr = expr.split(".")
        exprArr.pop()
        const newExpr = `${exprArr.join(".")}.fid`
        if (!foreignMap[newExpr] || i - foreignMap[newExpr] > 1) {
          foreignMap[newExpr] = i

          item.setting.fields.splice(i, 0, {
            id: `f${++maxNum}`,
            source: exprArr[exprArr.length - 1],
            shown: false,
            expr: newExpr,
            controlType: FOREIGN_REF,
          })

          i--
          continue
        } else {
          foreignMap[newExpr] = i
        }
      }
      item.setting.fieldMap[field.id] = field
      item.setting.fieldNameMap[field.label] = field
      field.vtype =
        field.controlType === NUMBER || field.dataType === NUMBER
          ? "num"
          : field.controlType === DATE || field.dataType === DATE
          ? "time"
          : "string"

      if (field.controlType === FOREIGN_REF) {
        item.setting.foreignRefMap[field.source] = field
        childMap[field.source] = childMap[field.source] || {}
        childMap[field.source][tableKey] = {
          label: item.label,
          name: item.name,
          shown: true,
          source: tableKey,
          controlType: CHILD_TABLE,
        }
      }
    }
  })

  Object.keys(childMap).forEach((tableKey) => {
    Object.keys(childMap[tableKey]).forEach((childKey) => {
      const item = childMap[tableKey][childKey]
      let id = `f${tableMap[tableKey].setting.fields.length}`
      item.id = id
      tableMap[tableKey].setting.fields.push(item)
      tableMap[tableKey].setting.fieldMap[item.id] = item
      item.label = tableMap[childKey].label
      item.name = tableMap[childKey].label
    })
  })
  // console.info('==========parseTables result', result);
  return result
}

module.exports = {
  getTables(tables = []) {
    return parseTables(tables)
  },
}
